# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Co57(KaitaiStruct):
    """:field ut: ut
    :field beacon_type: beacon_types.type_check.beacon_type
    :field time_counter: beacon_types.type_check.time_counter
    :field uplink_counter: beacon_types.type_check.uplink_counter
    :field camera_counter: beacon_types.type_check.camera_counter
    :field sel_reset_counter: beacon_types.type_check.sel_reset_counter
    :field antenna_deployed: beacon_types.type_check.antenna_deployed
    :field cw_duty_ratio: beacon_types.type_check.cw_duty_ratio
    :field obc_reset: beacon_types.type_check.obc_reset
    :field state_of_charge: beacon_types.type_check.state_of_charge
    :field state_of_obc: beacon_types.type_check.state_of_obc
    :field state_of_tx_tnc: beacon_types.type_check.state_of_tx_tnc
    :field undefined: beacon_types.type_check.undefined
    :field rssi_max_between_ut1_and_ut3: beacon_types.type_check.rssi_max_between_ut1_and_ut3
    :field batt_v: beacon_types.type_check.batt_v
    :field sol_v: beacon_types.type_check.sol_v
    :field batt_t_ut4: beacon_types.type_check.batt_t_ut4
    :field plus_x_i: beacon_types.type_check.plus_x_i
    :field minus_x_i: beacon_types.type_check.minus_x_i
    :field plus_y_i: beacon_types.type_check.plus_y_i
    :field minus_y_i: beacon_types.type_check.minus_y_i
    :field plus_z_i: beacon_types.type_check.plus_z_i
    :field minus_z_i: beacon_types.type_check.minus_z_i
    :field plus_x_t: beacon_types.type_check.plus_x_t
    :field minus_x_t: beacon_types.type_check.minus_x_t
    :field plus_y_t: beacon_types.type_check.plus_y_t
    :field minus_y_t: beacon_types.type_check.minus_y_t
    :field plus_z_t: beacon_types.type_check.plus_z_t
    :field minus_z_t: beacon_types.type_check.minus_z_t
    :field battery_t_ut6: beacon_types.type_check.battery_t_ut6
    :field fm_transmitter_t: beacon_types.type_check.fm_transmitter_t
    :field rssi_max_between_ut4_and_ut6: beacon_types.type_check.rssi_max_between_ut4_and_ut6
    :field beacon: beacon_types.type_check.beacon
    :field discard: beacon_types.type_check.discard_discard
    
    .. seealso::
       Source - https://web.archive.org/web/20040826163856fw_/http://www.space.t.u-tokyo.ac.jp/nlab_gs/xi-iv_operation/satelliteinfo_ja.html
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ut = (self._io.read_bytes(2)).decode(u"ASCII")
        if not self.ut == u"ut":
            raise kaitaistruct.ValidationNotEqualError(u"ut", self.ut, self._io, u"/seq/0")
        self.beacon_types = Co57.BeaconTypesT(self._io, self, self._root)

    class Ut5(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.plus_x_i_upper_four_bits = self._io.read_bits_int_be(4)
            self.minus_x_i_upper_four_bits = self._io.read_bits_int_be(4)
            self.plus_y_i_upper_four_bits = self._io.read_bits_int_be(4)
            self.minus_y_i_upper_four_bits = self._io.read_bits_int_be(4)
            self.plus_z_i_upper_four_bits = self._io.read_bits_int_be(4)
            self.minus_z_i_upper_four_bits = self._io.read_bits_int_be(4)
            self._io.align_to_byte()
            self.discard = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def value1(self):
            if hasattr(self, '_m_value1'):
                return self._m_value1

            self._m_value1 = (self.plus_x_i_upper_four_bits << 4)
            return getattr(self, '_m_value1', None)

        @property
        def value6_hex_left(self):
            if hasattr(self, '_m_value6_hex_left'):
                return self._m_value6_hex_left

            self._m_value6_hex_left = self.value6 // 16
            return getattr(self, '_m_value6_hex_left', None)

        @property
        def plus_z_i(self):
            if hasattr(self, '_m_plus_z_i'):
                return self._m_plus_z_i

            if  (((((((self.plus_x_i_upper_four_bits + self.minus_x_i_upper_four_bits) + self.plus_y_i_upper_four_bits) + self.minus_y_i_upper_four_bits) + self.plus_z_i_upper_four_bits) + self.minus_z_i_upper_four_bits) != 90) and ((((((self.plus_x_i_upper_four_bits + self.minus_x_i_upper_four_bits) + self.plus_y_i_upper_four_bits) + self.minus_y_i_upper_four_bits) + self.plus_z_i_upper_four_bits) + self.minus_z_i_upper_four_bits) != 0)) :
                self._m_plus_z_i = (((((((-0.636795098 * (self.minus_x_i_upper_four_bits << 4)) * 4.77) * 10) / 255) - ((((1.050451843 * (self.plus_y_i_upper_four_bits << 4)) * 4.77) * 10) / 255)) - ((((0.023700861 * (self.minus_y_i_upper_four_bits << 4)) * 4.77) * 10) / 255)) + ((((8.464182598 * (self.plus_z_i_upper_four_bits << 4)) * 4.77) * 10) / 255))

            return getattr(self, '_m_plus_z_i', None)

        @property
        def value1_hex_left(self):
            if hasattr(self, '_m_value1_hex_left'):
                return self._m_value1_hex_left

            self._m_value1_hex_left = self.value1 // 16
            return getattr(self, '_m_value1_hex_left', None)

        @property
        def value5_hex_left_digit(self):
            if hasattr(self, '_m_value5_hex_left_digit'):
                return self._m_value5_hex_left_digit

            self._m_value5_hex_left_digit = (u"a" if str(self.value5_hex_left) == u"10" else (u"b" if str(self.value5_hex_left) == u"11" else (u"c" if str(self.value5_hex_left) == u"12" else (u"d" if str(self.value5_hex_left) == u"13" else (u"e" if str(self.value5_hex_left) == u"14" else (u"f" if str(self.value5_hex_left) == u"15" else str(self.value5_hex_left)))))))
            return getattr(self, '_m_value5_hex_left_digit', None)

        @property
        def value3_hex(self):
            if hasattr(self, '_m_value3_hex'):
                return self._m_value3_hex

            self._m_value3_hex = self.value3_hex_left_digit
            return getattr(self, '_m_value3_hex', None)

        @property
        def beacon(self):
            if hasattr(self, '_m_beacon'):
                return self._m_beacon

            self._m_beacon = u"UT5 " + self.value1_hex + self.value2_hex + self.value3_hex + self.value4_hex + self.value5_hex + self.value6_hex
            return getattr(self, '_m_beacon', None)

        @property
        def value5_hex(self):
            if hasattr(self, '_m_value5_hex'):
                return self._m_value5_hex

            self._m_value5_hex = self.value5_hex_left_digit
            return getattr(self, '_m_value5_hex', None)

        @property
        def value4(self):
            if hasattr(self, '_m_value4'):
                return self._m_value4

            self._m_value4 = (self.minus_y_i_upper_four_bits << 4)
            return getattr(self, '_m_value4', None)

        @property
        def discard_discard(self):
            if hasattr(self, '_m_discard_discard'):
                return self._m_discard_discard

            if len(self.discard) != 0:
                self._m_discard_discard = int(self.discard) // 0

            return getattr(self, '_m_discard_discard', None)

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type

            self._m_beacon_type = 5
            return getattr(self, '_m_beacon_type', None)

        @property
        def value2(self):
            if hasattr(self, '_m_value2'):
                return self._m_value2

            self._m_value2 = (self.minus_x_i_upper_four_bits << 4)
            return getattr(self, '_m_value2', None)

        @property
        def value3_hex_left(self):
            if hasattr(self, '_m_value3_hex_left'):
                return self._m_value3_hex_left

            self._m_value3_hex_left = self.value3 // 16
            return getattr(self, '_m_value3_hex_left', None)

        @property
        def value5_hex_left(self):
            if hasattr(self, '_m_value5_hex_left'):
                return self._m_value5_hex_left

            self._m_value5_hex_left = self.value5 // 16
            return getattr(self, '_m_value5_hex_left', None)

        @property
        def value6(self):
            if hasattr(self, '_m_value6'):
                return self._m_value6

            self._m_value6 = (self.minus_z_i_upper_four_bits << 4)
            return getattr(self, '_m_value6', None)

        @property
        def minus_y_i(self):
            if hasattr(self, '_m_minus_y_i'):
                return self._m_minus_y_i

            if  (((((((self.plus_x_i_upper_four_bits + self.minus_x_i_upper_four_bits) + self.plus_y_i_upper_four_bits) + self.minus_y_i_upper_four_bits) + self.plus_z_i_upper_four_bits) + self.minus_z_i_upper_four_bits) != 90) and ((((((self.plus_x_i_upper_four_bits + self.minus_x_i_upper_four_bits) + self.plus_y_i_upper_four_bits) + self.minus_y_i_upper_four_bits) + self.plus_z_i_upper_four_bits) + self.minus_z_i_upper_four_bits) != 0)) :
                self._m_minus_y_i = (((((((-0.024941879 * (self.minus_x_i_upper_four_bits << 4)) * 4.77) * 10) / 255) - ((((0.017974511 * (self.plus_y_i_upper_four_bits << 4)) * 4.77) * 10) / 255)) + ((((9.840646080 * (self.minus_y_i_upper_four_bits << 4)) * 4.77) * 10) / 255)) - ((((0.023700861 * (self.plus_z_i_upper_four_bits << 4)) * 4.77) * 10) / 255))

            return getattr(self, '_m_minus_y_i', None)

        @property
        def value6_hex(self):
            if hasattr(self, '_m_value6_hex'):
                return self._m_value6_hex

            self._m_value6_hex = self.value6_hex_left_digit
            return getattr(self, '_m_value6_hex', None)

        @property
        def plus_x_i(self):
            if hasattr(self, '_m_plus_x_i'):
                return self._m_plus_x_i

            if  (((((((self.plus_x_i_upper_four_bits + self.minus_x_i_upper_four_bits) + self.plus_y_i_upper_four_bits) + self.minus_y_i_upper_four_bits) + self.plus_z_i_upper_four_bits) + self.minus_z_i_upper_four_bits) != 90) and ((((((self.plus_x_i_upper_four_bits + self.minus_x_i_upper_four_bits) + self.plus_y_i_upper_four_bits) + self.minus_y_i_upper_four_bits) + self.plus_z_i_upper_four_bits) + self.minus_z_i_upper_four_bits) != 0)) :
                self._m_plus_x_i = (((((self.plus_x_i_upper_four_bits << 4) * 4.77) * 10) * 10.49881) / 255)

            return getattr(self, '_m_plus_x_i', None)

        @property
        def plus_y_i(self):
            if hasattr(self, '_m_plus_y_i'):
                return self._m_plus_y_i

            if  (((((((self.plus_x_i_upper_four_bits + self.minus_x_i_upper_four_bits) + self.plus_y_i_upper_four_bits) + self.minus_y_i_upper_four_bits) + self.plus_z_i_upper_four_bits) + self.minus_z_i_upper_four_bits) != 90) and ((((((self.plus_x_i_upper_four_bits + self.minus_x_i_upper_four_bits) + self.plus_y_i_upper_four_bits) + self.minus_y_i_upper_four_bits) + self.plus_z_i_upper_four_bits) + self.minus_z_i_upper_four_bits) != 0)) :
                self._m_plus_y_i = (((((((-0.482939447 * (self.minus_x_i_upper_four_bits << 4)) * 4.77) * 10) / 255) + ((((6.672955921 * (self.plus_y_i_upper_four_bits << 4)) * 4.77) * 10) / 255)) - ((((0.017974511 * (self.minus_y_i_upper_four_bits << 4)) * 4.77) * 10) / 255)) - ((((1.050451843 * (self.plus_z_i_upper_four_bits << 4)) * 4.77) * 10) / 255))

            return getattr(self, '_m_plus_y_i', None)

        @property
        def value4_hex_left_digit(self):
            if hasattr(self, '_m_value4_hex_left_digit'):
                return self._m_value4_hex_left_digit

            self._m_value4_hex_left_digit = (u"a" if str(self.value4_hex_left) == u"10" else (u"b" if str(self.value4_hex_left) == u"11" else (u"c" if str(self.value4_hex_left) == u"12" else (u"d" if str(self.value4_hex_left) == u"13" else (u"e" if str(self.value4_hex_left) == u"14" else (u"f" if str(self.value4_hex_left) == u"15" else str(self.value4_hex_left)))))))
            return getattr(self, '_m_value4_hex_left_digit', None)

        @property
        def value3(self):
            if hasattr(self, '_m_value3'):
                return self._m_value3

            self._m_value3 = (self.plus_y_i_upper_four_bits << 4)
            return getattr(self, '_m_value3', None)

        @property
        def minus_z_i(self):
            if hasattr(self, '_m_minus_z_i'):
                return self._m_minus_z_i

            if  (((((((self.plus_x_i_upper_four_bits + self.minus_x_i_upper_four_bits) + self.plus_y_i_upper_four_bits) + self.minus_y_i_upper_four_bits) + self.plus_z_i_upper_four_bits) + self.minus_z_i_upper_four_bits) != 90) and ((((((self.plus_x_i_upper_four_bits + self.minus_x_i_upper_four_bits) + self.plus_y_i_upper_four_bits) + self.minus_y_i_upper_four_bits) + self.plus_z_i_upper_four_bits) + self.minus_z_i_upper_four_bits) != 0)) :
                self._m_minus_z_i = (7.515946019 * ((((self.minus_z_i_upper_four_bits << 4) * 4.77) * 10) / 255))

            return getattr(self, '_m_minus_z_i', None)

        @property
        def value1_hex(self):
            if hasattr(self, '_m_value1_hex'):
                return self._m_value1_hex

            self._m_value1_hex = self.value1_hex_left_digit
            return getattr(self, '_m_value1_hex', None)

        @property
        def value4_hex_left(self):
            if hasattr(self, '_m_value4_hex_left'):
                return self._m_value4_hex_left

            self._m_value4_hex_left = self.value4 // 16
            return getattr(self, '_m_value4_hex_left', None)

        @property
        def value2_hex_left_digit(self):
            if hasattr(self, '_m_value2_hex_left_digit'):
                return self._m_value2_hex_left_digit

            self._m_value2_hex_left_digit = (u"a" if str(self.value2_hex_left) == u"10" else (u"b" if str(self.value2_hex_left) == u"11" else (u"c" if str(self.value2_hex_left) == u"12" else (u"d" if str(self.value2_hex_left) == u"13" else (u"e" if str(self.value2_hex_left) == u"14" else (u"f" if str(self.value2_hex_left) == u"15" else str(self.value2_hex_left)))))))
            return getattr(self, '_m_value2_hex_left_digit', None)

        @property
        def value2_hex(self):
            if hasattr(self, '_m_value2_hex'):
                return self._m_value2_hex

            self._m_value2_hex = self.value2_hex_left_digit
            return getattr(self, '_m_value2_hex', None)

        @property
        def value4_hex(self):
            if hasattr(self, '_m_value4_hex'):
                return self._m_value4_hex

            self._m_value4_hex = self.value4_hex_left_digit
            return getattr(self, '_m_value4_hex', None)

        @property
        def value3_hex_left_digit(self):
            if hasattr(self, '_m_value3_hex_left_digit'):
                return self._m_value3_hex_left_digit

            self._m_value3_hex_left_digit = (u"a" if str(self.value3_hex_left) == u"10" else (u"b" if str(self.value3_hex_left) == u"11" else (u"c" if str(self.value3_hex_left) == u"12" else (u"d" if str(self.value3_hex_left) == u"13" else (u"e" if str(self.value3_hex_left) == u"14" else (u"f" if str(self.value3_hex_left) == u"15" else str(self.value3_hex_left)))))))
            return getattr(self, '_m_value3_hex_left_digit', None)

        @property
        def value6_hex_left_digit(self):
            if hasattr(self, '_m_value6_hex_left_digit'):
                return self._m_value6_hex_left_digit

            self._m_value6_hex_left_digit = (u"a" if str(self.value6_hex_left) == u"10" else (u"b" if str(self.value6_hex_left) == u"11" else (u"c" if str(self.value6_hex_left) == u"12" else (u"d" if str(self.value6_hex_left) == u"13" else (u"e" if str(self.value6_hex_left) == u"14" else (u"f" if str(self.value6_hex_left) == u"15" else str(self.value6_hex_left)))))))
            return getattr(self, '_m_value6_hex_left_digit', None)

        @property
        def value2_hex_left(self):
            if hasattr(self, '_m_value2_hex_left'):
                return self._m_value2_hex_left

            self._m_value2_hex_left = self.value2 // 16
            return getattr(self, '_m_value2_hex_left', None)

        @property
        def minus_x_i(self):
            if hasattr(self, '_m_minus_x_i'):
                return self._m_minus_x_i

            if  (((((((self.plus_x_i_upper_four_bits + self.minus_x_i_upper_four_bits) + self.plus_y_i_upper_four_bits) + self.minus_y_i_upper_four_bits) + self.plus_z_i_upper_four_bits) + self.minus_z_i_upper_four_bits) != 90) and ((((((self.plus_x_i_upper_four_bits + self.minus_x_i_upper_four_bits) + self.plus_y_i_upper_four_bits) + self.minus_y_i_upper_four_bits) + self.plus_z_i_upper_four_bits) + self.minus_z_i_upper_four_bits) != 0)) :
                self._m_minus_x_i = (((((((8.251522329 * (self.minus_x_i_upper_four_bits << 4)) * 4.77) * 10) / 255) - ((((0.482939447 * (self.plus_y_i_upper_four_bits << 4)) * 4.77) * 10) / 255)) - ((((0.024941879 * (self.minus_y_i_upper_four_bits << 4)) * 4.77) * 10) / 255)) - ((((0.636795098 * (self.plus_z_i_upper_four_bits << 4)) * 4.77) * 10) / 255))

            return getattr(self, '_m_minus_x_i', None)

        @property
        def value5(self):
            if hasattr(self, '_m_value5'):
                return self._m_value5

            self._m_value5 = (self.plus_z_i_upper_four_bits << 4)
            return getattr(self, '_m_value5', None)

        @property
        def value1_hex_left_digit(self):
            if hasattr(self, '_m_value1_hex_left_digit'):
                return self._m_value1_hex_left_digit

            self._m_value1_hex_left_digit = (u"a" if str(self.value1_hex_left) == u"10" else (u"b" if str(self.value1_hex_left) == u"11" else (u"c" if str(self.value1_hex_left) == u"12" else (u"d" if str(self.value1_hex_left) == u"13" else (u"e" if str(self.value1_hex_left) == u"14" else (u"f" if str(self.value1_hex_left) == u"15" else str(self.value1_hex_left)))))))
            return getattr(self, '_m_value1_hex_left_digit', None)


    class Ut4(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.batt_v_raw = self._io.read_u1()
            self.sol_v_raw = self._io.read_u1()
            self.batt_t_ut4_raw = self._io.read_u1()
            self.discard = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def value1(self):
            if hasattr(self, '_m_value1'):
                return self._m_value1

            self._m_value1 = self.batt_v_raw
            return getattr(self, '_m_value1', None)

        @property
        def value3_hex_right_digit(self):
            if hasattr(self, '_m_value3_hex_right_digit'):
                return self._m_value3_hex_right_digit

            self._m_value3_hex_right_digit = (u"a" if str(self.value3_hex_right) == u"10" else (u"b" if str(self.value3_hex_right) == u"11" else (u"c" if str(self.value3_hex_right) == u"12" else (u"d" if str(self.value3_hex_right) == u"13" else (u"e" if str(self.value3_hex_right) == u"14" else (u"f" if str(self.value3_hex_right) == u"15" else str(self.value3_hex_right)))))))
            return getattr(self, '_m_value3_hex_right_digit', None)

        @property
        def value1_hex_left(self):
            if hasattr(self, '_m_value1_hex_left'):
                return self._m_value1_hex_left

            self._m_value1_hex_left = self.value1 // 16
            return getattr(self, '_m_value1_hex_left', None)

        @property
        def value2_hex_right(self):
            if hasattr(self, '_m_value2_hex_right'):
                return self._m_value2_hex_right

            self._m_value2_hex_right = (self.value2 % 16)
            return getattr(self, '_m_value2_hex_right', None)

        @property
        def value3_hex(self):
            if hasattr(self, '_m_value3_hex'):
                return self._m_value3_hex

            self._m_value3_hex = self.value3_hex_left_digit + self.value3_hex_right_digit
            return getattr(self, '_m_value3_hex', None)

        @property
        def value3_hex_right(self):
            if hasattr(self, '_m_value3_hex_right'):
                return self._m_value3_hex_right

            self._m_value3_hex_right = (self.value3 % 16)
            return getattr(self, '_m_value3_hex_right', None)

        @property
        def value1_hex_right_digit(self):
            if hasattr(self, '_m_value1_hex_right_digit'):
                return self._m_value1_hex_right_digit

            self._m_value1_hex_right_digit = (u"a" if str(self.value1_hex_right) == u"10" else (u"b" if str(self.value1_hex_right) == u"11" else (u"c" if str(self.value1_hex_right) == u"12" else (u"d" if str(self.value1_hex_right) == u"13" else (u"e" if str(self.value1_hex_right) == u"14" else (u"f" if str(self.value1_hex_right) == u"15" else str(self.value1_hex_right)))))))
            return getattr(self, '_m_value1_hex_right_digit', None)

        @property
        def beacon(self):
            if hasattr(self, '_m_beacon'):
                return self._m_beacon

            self._m_beacon = u"UT4 " + self.value1_hex + self.value2_hex + self.value3_hex
            return getattr(self, '_m_beacon', None)

        @property
        def discard_discard(self):
            if hasattr(self, '_m_discard_discard'):
                return self._m_discard_discard

            if len(self.discard) != 0:
                self._m_discard_discard = int(self.discard) // 0

            return getattr(self, '_m_discard_discard', None)

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type

            self._m_beacon_type = 4
            return getattr(self, '_m_beacon_type', None)

        @property
        def value2(self):
            if hasattr(self, '_m_value2'):
                return self._m_value2

            self._m_value2 = self.sol_v_raw
            return getattr(self, '_m_value2', None)

        @property
        def value3_hex_left(self):
            if hasattr(self, '_m_value3_hex_left'):
                return self._m_value3_hex_left

            self._m_value3_hex_left = self.value3 // 16
            return getattr(self, '_m_value3_hex_left', None)

        @property
        def sol_v(self):
            if hasattr(self, '_m_sol_v'):
                return self._m_sol_v

            self._m_sol_v = (((self.sol_v_raw * 4.77) * 4) / 255)
            return getattr(self, '_m_sol_v', None)

        @property
        def batt_t_ut4(self):
            if hasattr(self, '_m_batt_t_ut4'):
                return self._m_batt_t_ut4

            self._m_batt_t_ut4 = ((((self.batt_t_ut4_raw + 297.06) / 149.31) * 105.86) - 287.71)
            return getattr(self, '_m_batt_t_ut4', None)

        @property
        def batt_v(self):
            if hasattr(self, '_m_batt_v'):
                return self._m_batt_v

            self._m_batt_v = (((self.batt_v_raw * 4.77) * 4) / 255)
            return getattr(self, '_m_batt_v', None)

        @property
        def value1_hex_right(self):
            if hasattr(self, '_m_value1_hex_right'):
                return self._m_value1_hex_right

            self._m_value1_hex_right = (self.value1 % 16)
            return getattr(self, '_m_value1_hex_right', None)

        @property
        def value3(self):
            if hasattr(self, '_m_value3'):
                return self._m_value3

            self._m_value3 = self.batt_t_ut4_raw
            return getattr(self, '_m_value3', None)

        @property
        def value1_hex(self):
            if hasattr(self, '_m_value1_hex'):
                return self._m_value1_hex

            self._m_value1_hex = self.value1_hex_left_digit + self.value1_hex_right_digit
            return getattr(self, '_m_value1_hex', None)

        @property
        def value2_hex_right_digit(self):
            if hasattr(self, '_m_value2_hex_right_digit'):
                return self._m_value2_hex_right_digit

            self._m_value2_hex_right_digit = (u"a" if str(self.value2_hex_right) == u"10" else (u"b" if str(self.value2_hex_right) == u"11" else (u"c" if str(self.value2_hex_right) == u"12" else (u"d" if str(self.value2_hex_right) == u"13" else (u"e" if str(self.value2_hex_right) == u"14" else (u"f" if str(self.value2_hex_right) == u"15" else str(self.value2_hex_right)))))))
            return getattr(self, '_m_value2_hex_right_digit', None)

        @property
        def value2_hex_left_digit(self):
            if hasattr(self, '_m_value2_hex_left_digit'):
                return self._m_value2_hex_left_digit

            self._m_value2_hex_left_digit = (u"a" if str(self.value2_hex_left) == u"10" else (u"b" if str(self.value2_hex_left) == u"11" else (u"c" if str(self.value2_hex_left) == u"12" else (u"d" if str(self.value2_hex_left) == u"13" else (u"e" if str(self.value2_hex_left) == u"14" else (u"f" if str(self.value2_hex_left) == u"15" else str(self.value2_hex_left)))))))
            return getattr(self, '_m_value2_hex_left_digit', None)

        @property
        def value2_hex(self):
            if hasattr(self, '_m_value2_hex'):
                return self._m_value2_hex

            self._m_value2_hex = self.value2_hex_left_digit + self.value2_hex_right_digit
            return getattr(self, '_m_value2_hex', None)

        @property
        def value3_hex_left_digit(self):
            if hasattr(self, '_m_value3_hex_left_digit'):
                return self._m_value3_hex_left_digit

            self._m_value3_hex_left_digit = (u"a" if str(self.value3_hex_left) == u"10" else (u"b" if str(self.value3_hex_left) == u"11" else (u"c" if str(self.value3_hex_left) == u"12" else (u"d" if str(self.value3_hex_left) == u"13" else (u"e" if str(self.value3_hex_left) == u"14" else (u"f" if str(self.value3_hex_left) == u"15" else str(self.value3_hex_left)))))))
            return getattr(self, '_m_value3_hex_left_digit', None)

        @property
        def value2_hex_left(self):
            if hasattr(self, '_m_value2_hex_left'):
                return self._m_value2_hex_left

            self._m_value2_hex_left = self.value2 // 16
            return getattr(self, '_m_value2_hex_left', None)

        @property
        def value1_hex_left_digit(self):
            if hasattr(self, '_m_value1_hex_left_digit'):
                return self._m_value1_hex_left_digit

            self._m_value1_hex_left_digit = (u"a" if str(self.value1_hex_left) == u"10" else (u"b" if str(self.value1_hex_left) == u"11" else (u"c" if str(self.value1_hex_left) == u"12" else (u"d" if str(self.value1_hex_left) == u"13" else (u"e" if str(self.value1_hex_left) == u"14" else (u"f" if str(self.value1_hex_left) == u"15" else str(self.value1_hex_left)))))))
            return getattr(self, '_m_value1_hex_left_digit', None)


    class Discard(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.discard = self._io.read_bits_int_be(1) != 0

        @property
        def discard_discard(self):
            if hasattr(self, '_m_discard_discard'):
                return self._m_discard_discard

            self._m_discard_discard = int(self.discard) // 0
            return getattr(self, '_m_discard_discard', None)


    class Ut3(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.byte_1 = self._io.read_u1()
            self.byte_2 = self._io.read_u1()
            self.byte_3 = self._io.read_u1()
            self.rssi_max_between_ut1_and_ut3_raw = self._io.read_u1()
            self.discard = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def value1(self):
            if hasattr(self, '_m_value1'):
                return self._m_value1

            self._m_value1 = self.byte_1
            return getattr(self, '_m_value1', None)

        @property
        def value3_hex_right_digit(self):
            if hasattr(self, '_m_value3_hex_right_digit'):
                return self._m_value3_hex_right_digit

            self._m_value3_hex_right_digit = (u"a" if str(self.value3_hex_right) == u"10" else (u"b" if str(self.value3_hex_right) == u"11" else (u"c" if str(self.value3_hex_right) == u"12" else (u"d" if str(self.value3_hex_right) == u"13" else (u"e" if str(self.value3_hex_right) == u"14" else (u"f" if str(self.value3_hex_right) == u"15" else str(self.value3_hex_right)))))))
            return getattr(self, '_m_value3_hex_right_digit', None)

        @property
        def value1_hex_left(self):
            if hasattr(self, '_m_value1_hex_left'):
                return self._m_value1_hex_left

            self._m_value1_hex_left = self.value1 // 16
            return getattr(self, '_m_value1_hex_left', None)

        @property
        def value2_hex_right(self):
            if hasattr(self, '_m_value2_hex_right'):
                return self._m_value2_hex_right

            self._m_value2_hex_right = (self.value2 % 16)
            return getattr(self, '_m_value2_hex_right', None)

        @property
        def value3_hex(self):
            if hasattr(self, '_m_value3_hex'):
                return self._m_value3_hex

            self._m_value3_hex = self.value3_hex_left_digit + self.value3_hex_right_digit
            return getattr(self, '_m_value3_hex', None)

        @property
        def value3_hex_right(self):
            if hasattr(self, '_m_value3_hex_right'):
                return self._m_value3_hex_right

            self._m_value3_hex_right = (self.value3 % 16)
            return getattr(self, '_m_value3_hex_right', None)

        @property
        def rssi_max_between_ut1_and_ut3(self):
            if hasattr(self, '_m_rssi_max_between_ut1_and_ut3'):
                return self._m_rssi_max_between_ut1_and_ut3

            self._m_rssi_max_between_ut1_and_ut3 = ((((self.rssi_max_between_ut1_and_ut3_raw * 4.77) * 4) / 255) - 107)
            return getattr(self, '_m_rssi_max_between_ut1_and_ut3', None)

        @property
        def value1_hex_right_digit(self):
            if hasattr(self, '_m_value1_hex_right_digit'):
                return self._m_value1_hex_right_digit

            self._m_value1_hex_right_digit = (u"a" if str(self.value1_hex_right) == u"10" else (u"b" if str(self.value1_hex_right) == u"11" else (u"c" if str(self.value1_hex_right) == u"12" else (u"d" if str(self.value1_hex_right) == u"13" else (u"e" if str(self.value1_hex_right) == u"14" else (u"f" if str(self.value1_hex_right) == u"15" else str(self.value1_hex_right)))))))
            return getattr(self, '_m_value1_hex_right_digit', None)

        @property
        def beacon(self):
            if hasattr(self, '_m_beacon'):
                return self._m_beacon

            self._m_beacon = u"UT3 " + self.value1_hex + self.value2_hex + self.value3_hex + self.value4_hex
            return getattr(self, '_m_beacon', None)

        @property
        def sel_reset_counter(self):
            if hasattr(self, '_m_sel_reset_counter'):
                return self._m_sel_reset_counter

            self._m_sel_reset_counter = (self.byte_2 & 7)
            return getattr(self, '_m_sel_reset_counter', None)

        @property
        def obc_reset(self):
            if hasattr(self, '_m_obc_reset'):
                return self._m_obc_reset

            self._m_obc_reset = ((self.byte_2 >> 6) & 1)
            return getattr(self, '_m_obc_reset', None)

        @property
        def value4(self):
            if hasattr(self, '_m_value4'):
                return self._m_value4

            self._m_value4 = self.rssi_max_between_ut1_and_ut3_raw
            return getattr(self, '_m_value4', None)

        @property
        def discard_discard(self):
            if hasattr(self, '_m_discard_discard'):
                return self._m_discard_discard

            if len(self.discard) != 0:
                self._m_discard_discard = int(self.discard) // 0

            return getattr(self, '_m_discard_discard', None)

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type

            self._m_beacon_type = 3
            return getattr(self, '_m_beacon_type', None)

        @property
        def value2(self):
            if hasattr(self, '_m_value2'):
                return self._m_value2

            self._m_value2 = self.byte_2
            return getattr(self, '_m_value2', None)

        @property
        def value3_hex_left(self):
            if hasattr(self, '_m_value3_hex_left'):
                return self._m_value3_hex_left

            self._m_value3_hex_left = self.value3 // 16
            return getattr(self, '_m_value3_hex_left', None)

        @property
        def state_of_charge(self):
            if hasattr(self, '_m_state_of_charge'):
                return self._m_state_of_charge

            self._m_state_of_charge = (self.byte_2 >> 7)
            return getattr(self, '_m_state_of_charge', None)

        @property
        def antenna_deployed(self):
            if hasattr(self, '_m_antenna_deployed'):
                return self._m_antenna_deployed

            self._m_antenna_deployed = ((self.byte_2 >> 3) & 1)
            return getattr(self, '_m_antenna_deployed', None)

        @property
        def undefined(self):
            if hasattr(self, '_m_undefined'):
                return self._m_undefined

            self._m_undefined = (self.byte_3 >> 2)
            return getattr(self, '_m_undefined', None)

        @property
        def value4_hex_left_digit(self):
            if hasattr(self, '_m_value4_hex_left_digit'):
                return self._m_value4_hex_left_digit

            self._m_value4_hex_left_digit = (u"a" if str(self.value4_hex_left) == u"10" else (u"b" if str(self.value4_hex_left) == u"11" else (u"c" if str(self.value4_hex_left) == u"12" else (u"d" if str(self.value4_hex_left) == u"13" else (u"e" if str(self.value4_hex_left) == u"14" else (u"f" if str(self.value4_hex_left) == u"15" else str(self.value4_hex_left)))))))
            return getattr(self, '_m_value4_hex_left_digit', None)

        @property
        def value1_hex_right(self):
            if hasattr(self, '_m_value1_hex_right'):
                return self._m_value1_hex_right

            self._m_value1_hex_right = (self.value1 % 16)
            return getattr(self, '_m_value1_hex_right', None)

        @property
        def uplink_counter(self):
            if hasattr(self, '_m_uplink_counter'):
                return self._m_uplink_counter

            self._m_uplink_counter = (self.byte_1 & 31)
            return getattr(self, '_m_uplink_counter', None)

        @property
        def value4_hex_right(self):
            if hasattr(self, '_m_value4_hex_right'):
                return self._m_value4_hex_right

            self._m_value4_hex_right = (self.value4 % 16)
            return getattr(self, '_m_value4_hex_right', None)

        @property
        def value3(self):
            if hasattr(self, '_m_value3'):
                return self._m_value3

            self._m_value3 = self.byte_3
            return getattr(self, '_m_value3', None)

        @property
        def value4_hex_right_digit(self):
            if hasattr(self, '_m_value4_hex_right_digit'):
                return self._m_value4_hex_right_digit

            self._m_value4_hex_right_digit = (u"a" if str(self.value4_hex_right) == u"10" else (u"b" if str(self.value4_hex_right) == u"11" else (u"c" if str(self.value4_hex_right) == u"12" else (u"d" if str(self.value4_hex_right) == u"13" else (u"e" if str(self.value4_hex_right) == u"14" else (u"f" if str(self.value4_hex_right) == u"15" else str(self.value4_hex_right)))))))
            return getattr(self, '_m_value4_hex_right_digit', None)

        @property
        def value1_hex(self):
            if hasattr(self, '_m_value1_hex'):
                return self._m_value1_hex

            self._m_value1_hex = self.value1_hex_left_digit + self.value1_hex_right_digit
            return getattr(self, '_m_value1_hex', None)

        @property
        def value2_hex_right_digit(self):
            if hasattr(self, '_m_value2_hex_right_digit'):
                return self._m_value2_hex_right_digit

            self._m_value2_hex_right_digit = (u"a" if str(self.value2_hex_right) == u"10" else (u"b" if str(self.value2_hex_right) == u"11" else (u"c" if str(self.value2_hex_right) == u"12" else (u"d" if str(self.value2_hex_right) == u"13" else (u"e" if str(self.value2_hex_right) == u"14" else (u"f" if str(self.value2_hex_right) == u"15" else str(self.value2_hex_right)))))))
            return getattr(self, '_m_value2_hex_right_digit', None)

        @property
        def value4_hex_left(self):
            if hasattr(self, '_m_value4_hex_left'):
                return self._m_value4_hex_left

            self._m_value4_hex_left = self.value4 // 16
            return getattr(self, '_m_value4_hex_left', None)

        @property
        def value2_hex_left_digit(self):
            if hasattr(self, '_m_value2_hex_left_digit'):
                return self._m_value2_hex_left_digit

            self._m_value2_hex_left_digit = (u"a" if str(self.value2_hex_left) == u"10" else (u"b" if str(self.value2_hex_left) == u"11" else (u"c" if str(self.value2_hex_left) == u"12" else (u"d" if str(self.value2_hex_left) == u"13" else (u"e" if str(self.value2_hex_left) == u"14" else (u"f" if str(self.value2_hex_left) == u"15" else str(self.value2_hex_left)))))))
            return getattr(self, '_m_value2_hex_left_digit', None)

        @property
        def value2_hex(self):
            if hasattr(self, '_m_value2_hex'):
                return self._m_value2_hex

            self._m_value2_hex = self.value2_hex_left_digit + self.value2_hex_right_digit
            return getattr(self, '_m_value2_hex', None)

        @property
        def value4_hex(self):
            if hasattr(self, '_m_value4_hex'):
                return self._m_value4_hex

            self._m_value4_hex = self.value4_hex_left_digit + self.value4_hex_right_digit
            return getattr(self, '_m_value4_hex', None)

        @property
        def cw_duty_ratio(self):
            if hasattr(self, '_m_cw_duty_ratio'):
                return self._m_cw_duty_ratio

            self._m_cw_duty_ratio = ((self.byte_2 >> 4) & 3)
            return getattr(self, '_m_cw_duty_ratio', None)

        @property
        def value3_hex_left_digit(self):
            if hasattr(self, '_m_value3_hex_left_digit'):
                return self._m_value3_hex_left_digit

            self._m_value3_hex_left_digit = (u"a" if str(self.value3_hex_left) == u"10" else (u"b" if str(self.value3_hex_left) == u"11" else (u"c" if str(self.value3_hex_left) == u"12" else (u"d" if str(self.value3_hex_left) == u"13" else (u"e" if str(self.value3_hex_left) == u"14" else (u"f" if str(self.value3_hex_left) == u"15" else str(self.value3_hex_left)))))))
            return getattr(self, '_m_value3_hex_left_digit', None)

        @property
        def camera_counter(self):
            if hasattr(self, '_m_camera_counter'):
                return self._m_camera_counter

            self._m_camera_counter = (self.byte_1 >> 5)
            return getattr(self, '_m_camera_counter', None)

        @property
        def value2_hex_left(self):
            if hasattr(self, '_m_value2_hex_left'):
                return self._m_value2_hex_left

            self._m_value2_hex_left = self.value2 // 16
            return getattr(self, '_m_value2_hex_left', None)

        @property
        def state_of_tx_tnc(self):
            if hasattr(self, '_m_state_of_tx_tnc'):
                return self._m_state_of_tx_tnc

            self._m_state_of_tx_tnc = ((self.byte_3 >> 1) & 1)
            return getattr(self, '_m_state_of_tx_tnc', None)

        @property
        def value1_hex_left_digit(self):
            if hasattr(self, '_m_value1_hex_left_digit'):
                return self._m_value1_hex_left_digit

            self._m_value1_hex_left_digit = (u"a" if str(self.value1_hex_left) == u"10" else (u"b" if str(self.value1_hex_left) == u"11" else (u"c" if str(self.value1_hex_left) == u"12" else (u"d" if str(self.value1_hex_left) == u"13" else (u"e" if str(self.value1_hex_left) == u"14" else (u"f" if str(self.value1_hex_left) == u"15" else str(self.value1_hex_left)))))))
            return getattr(self, '_m_value1_hex_left_digit', None)

        @property
        def state_of_obc(self):
            if hasattr(self, '_m_state_of_obc'):
                return self._m_state_of_obc

            self._m_state_of_obc = (self.byte_3 & 1)
            return getattr(self, '_m_state_of_obc', None)


    class BeaconTypesT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check
            if _on == 13856:
                self.type_check = Co57.Ut6(self._io, self, self._root)
            elif _on == 13600:
                self.type_check = Co57.Ut5(self._io, self, self._root)
            elif _on == 12832:
                self.type_check = Co57.Ut2(self._io, self, self._root)
            elif _on == 13344:
                self.type_check = Co57.Ut4(self._io, self, self._root)
            elif _on == 13088:
                self.type_check = Co57.Ut3(self._io, self, self._root)
            else:
                self.type_check = Co57.Discard(self._io, self, self._root)

        @property
        def check(self):
            if hasattr(self, '_m_check'):
                return self._m_check

            self._m_check = self._io.read_u2be()
            return getattr(self, '_m_check', None)


    class Ut6(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.plus_x_t_upper_four_bits = self._io.read_bits_int_be(4)
            self.minus_x_t_upper_four_bits = self._io.read_bits_int_be(4)
            self.plus_y_t_upper_four_bits = self._io.read_bits_int_be(4)
            self.minus_y_t_upper_four_bits = self._io.read_bits_int_be(4)
            self.plus_z_t_upper_four_bits = self._io.read_bits_int_be(4)
            self.minus_z_t_upper_four_bits = self._io.read_bits_int_be(4)
            self.battery_t_upper_four_bits = self._io.read_bits_int_be(4)
            self.fm_transmitter_t_upper_four_bits = self._io.read_bits_int_be(4)
            self._io.align_to_byte()
            self.rssi_max_between_ut4_and_ut6_raw = self._io.read_u1()
            self.discard = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def value1(self):
            if hasattr(self, '_m_value1'):
                return self._m_value1

            self._m_value1 = (self.plus_x_t_upper_four_bits << 4)
            return getattr(self, '_m_value1', None)

        @property
        def value6_hex_left(self):
            if hasattr(self, '_m_value6_hex_left'):
                return self._m_value6_hex_left

            self._m_value6_hex_left = self.value6 // 16
            return getattr(self, '_m_value6_hex_left', None)

        @property
        def battery_t_ut6(self):
            if hasattr(self, '_m_battery_t_ut6'):
                return self._m_battery_t_ut6

            self._m_battery_t_ut6 = (((((self.battery_t_upper_four_bits << 4) + 297.06) / 149.31) * 105.86) - 287.71)
            return getattr(self, '_m_battery_t_ut6', None)

        @property
        def value1_hex_left(self):
            if hasattr(self, '_m_value1_hex_left'):
                return self._m_value1_hex_left

            self._m_value1_hex_left = self.value1 // 16
            return getattr(self, '_m_value1_hex_left', None)

        @property
        def value9_hex(self):
            if hasattr(self, '_m_value9_hex'):
                return self._m_value9_hex

            self._m_value9_hex = self.value9_hex_left_digit + self.value9_hex_right_digit
            return getattr(self, '_m_value9_hex', None)

        @property
        def value8_hex_left(self):
            if hasattr(self, '_m_value8_hex_left'):
                return self._m_value8_hex_left

            self._m_value8_hex_left = self.value8 // 16
            return getattr(self, '_m_value8_hex_left', None)

        @property
        def value5_hex_left_digit(self):
            if hasattr(self, '_m_value5_hex_left_digit'):
                return self._m_value5_hex_left_digit

            self._m_value5_hex_left_digit = (u"a" if str(self.value5_hex_left) == u"10" else (u"b" if str(self.value5_hex_left) == u"11" else (u"c" if str(self.value5_hex_left) == u"12" else (u"d" if str(self.value5_hex_left) == u"13" else (u"e" if str(self.value5_hex_left) == u"14" else (u"f" if str(self.value5_hex_left) == u"15" else str(self.value5_hex_left)))))))
            return getattr(self, '_m_value5_hex_left_digit', None)

        @property
        def minus_z_t(self):
            if hasattr(self, '_m_minus_z_t'):
                return self._m_minus_z_t

            self._m_minus_z_t = (((((self.minus_z_t_upper_four_bits << 4) + 297.81) / 149.55) * 106.57) - 289.76)
            return getattr(self, '_m_minus_z_t', None)

        @property
        def value9(self):
            if hasattr(self, '_m_value9'):
                return self._m_value9

            self._m_value9 = self.rssi_max_between_ut4_and_ut6_raw
            return getattr(self, '_m_value9', None)

        @property
        def plus_z_t(self):
            if hasattr(self, '_m_plus_z_t'):
                return self._m_plus_z_t

            self._m_plus_z_t = (((((self.minus_x_t_upper_four_bits << 4) + 298.51) / 149.75) * 104.55) - 285.08)
            return getattr(self, '_m_plus_z_t', None)

        @property
        def value3_hex(self):
            if hasattr(self, '_m_value3_hex'):
                return self._m_value3_hex

            self._m_value3_hex = self.value3_hex_left_digit
            return getattr(self, '_m_value3_hex', None)

        @property
        def beacon(self):
            if hasattr(self, '_m_beacon'):
                return self._m_beacon

            self._m_beacon = u"UT6 " + self.value1_hex + self.value2_hex + self.value3_hex + self.value4_hex + self.value5_hex + self.value6_hex + self.value7_hex + self.value8_hex + self.value9_hex
            return getattr(self, '_m_beacon', None)

        @property
        def value5_hex(self):
            if hasattr(self, '_m_value5_hex'):
                return self._m_value5_hex

            self._m_value5_hex = self.value5_hex_left_digit
            return getattr(self, '_m_value5_hex', None)

        @property
        def value7_hex_left_digit(self):
            if hasattr(self, '_m_value7_hex_left_digit'):
                return self._m_value7_hex_left_digit

            self._m_value7_hex_left_digit = (u"a" if str(self.value7_hex_left) == u"10" else (u"b" if str(self.value7_hex_left) == u"11" else (u"c" if str(self.value7_hex_left) == u"12" else (u"d" if str(self.value7_hex_left) == u"13" else (u"e" if str(self.value7_hex_left) == u"14" else (u"f" if str(self.value7_hex_left) == u"15" else str(self.value7_hex_left)))))))
            return getattr(self, '_m_value7_hex_left_digit', None)

        @property
        def value4(self):
            if hasattr(self, '_m_value4'):
                return self._m_value4

            self._m_value4 = (self.minus_y_t_upper_four_bits << 4)
            return getattr(self, '_m_value4', None)

        @property
        def discard_discard(self):
            if hasattr(self, '_m_discard_discard'):
                return self._m_discard_discard

            if len(self.discard) != 0:
                self._m_discard_discard = int(self.discard) // 0

            return getattr(self, '_m_discard_discard', None)

        @property
        def value9_hex_left(self):
            if hasattr(self, '_m_value9_hex_left'):
                return self._m_value9_hex_left

            self._m_value9_hex_left = self.value9 // 16
            return getattr(self, '_m_value9_hex_left', None)

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type

            self._m_beacon_type = 6
            return getattr(self, '_m_beacon_type', None)

        @property
        def value2(self):
            if hasattr(self, '_m_value2'):
                return self._m_value2

            self._m_value2 = (self.minus_x_t_upper_four_bits << 4)
            return getattr(self, '_m_value2', None)

        @property
        def value3_hex_left(self):
            if hasattr(self, '_m_value3_hex_left'):
                return self._m_value3_hex_left

            self._m_value3_hex_left = self.value3 // 16
            return getattr(self, '_m_value3_hex_left', None)

        @property
        def value5_hex_left(self):
            if hasattr(self, '_m_value5_hex_left'):
                return self._m_value5_hex_left

            self._m_value5_hex_left = self.value5 // 16
            return getattr(self, '_m_value5_hex_left', None)

        @property
        def value6(self):
            if hasattr(self, '_m_value6'):
                return self._m_value6

            self._m_value6 = (self.minus_z_t_upper_four_bits << 4)
            return getattr(self, '_m_value6', None)

        @property
        def value8_hex_left_digit(self):
            if hasattr(self, '_m_value8_hex_left_digit'):
                return self._m_value8_hex_left_digit

            self._m_value8_hex_left_digit = (u"a" if str(self.value8_hex_left) == u"10" else (u"b" if str(self.value8_hex_left) == u"11" else (u"c" if str(self.value8_hex_left) == u"12" else (u"d" if str(self.value8_hex_left) == u"13" else (u"e" if str(self.value8_hex_left) == u"14" else (u"f" if str(self.value8_hex_left) == u"15" else str(self.value8_hex_left)))))))
            return getattr(self, '_m_value8_hex_left_digit', None)

        @property
        def value6_hex(self):
            if hasattr(self, '_m_value6_hex'):
                return self._m_value6_hex

            self._m_value6_hex = self.value6_hex_left_digit
            return getattr(self, '_m_value6_hex', None)

        @property
        def plus_x_t(self):
            if hasattr(self, '_m_plus_x_t'):
                return self._m_plus_x_t

            self._m_plus_x_t = (((((self.plus_x_t_upper_four_bits << 4) + 298.43) / 149.66) * 105.25) - 287.12)
            return getattr(self, '_m_plus_x_t', None)

        @property
        def value4_hex_left_digit(self):
            if hasattr(self, '_m_value4_hex_left_digit'):
                return self._m_value4_hex_left_digit

            self._m_value4_hex_left_digit = (u"a" if str(self.value4_hex_left) == u"10" else (u"b" if str(self.value4_hex_left) == u"11" else (u"c" if str(self.value4_hex_left) == u"12" else (u"d" if str(self.value4_hex_left) == u"13" else (u"e" if str(self.value4_hex_left) == u"14" else (u"f" if str(self.value4_hex_left) == u"15" else str(self.value4_hex_left)))))))
            return getattr(self, '_m_value4_hex_left_digit', None)

        @property
        def value3(self):
            if hasattr(self, '_m_value3'):
                return self._m_value3

            self._m_value3 = (self.plus_y_t_upper_four_bits << 4)
            return getattr(self, '_m_value3', None)

        @property
        def value1_hex(self):
            if hasattr(self, '_m_value1_hex'):
                return self._m_value1_hex

            self._m_value1_hex = self.value1_hex_left_digit
            return getattr(self, '_m_value1_hex', None)

        @property
        def minus_y_t(self):
            if hasattr(self, '_m_minus_y_t'):
                return self._m_minus_y_t

            self._m_minus_y_t = (((((self.minus_x_t_upper_four_bits << 4) + 297.80) / 149.51) * 106.16) - 288.55)
            return getattr(self, '_m_minus_y_t', None)

        @property
        def rssi_max_between_ut4_and_ut6(self):
            if hasattr(self, '_m_rssi_max_between_ut4_and_ut6'):
                return self._m_rssi_max_between_ut4_and_ut6

            self._m_rssi_max_between_ut4_and_ut6 = ((((self.rssi_max_between_ut4_and_ut6_raw * 4.77) * 4) / 255) - 107)
            return getattr(self, '_m_rssi_max_between_ut4_and_ut6', None)

        @property
        def value7_hex_left(self):
            if hasattr(self, '_m_value7_hex_left'):
                return self._m_value7_hex_left

            self._m_value7_hex_left = self.value7 // 16
            return getattr(self, '_m_value7_hex_left', None)

        @property
        def value7(self):
            if hasattr(self, '_m_value7'):
                return self._m_value7

            self._m_value7 = (self.battery_t_upper_four_bits << 4)
            return getattr(self, '_m_value7', None)

        @property
        def value4_hex_left(self):
            if hasattr(self, '_m_value4_hex_left'):
                return self._m_value4_hex_left

            self._m_value4_hex_left = self.value4 // 16
            return getattr(self, '_m_value4_hex_left', None)

        @property
        def value2_hex_left_digit(self):
            if hasattr(self, '_m_value2_hex_left_digit'):
                return self._m_value2_hex_left_digit

            self._m_value2_hex_left_digit = (u"a" if str(self.value2_hex_left) == u"10" else (u"b" if str(self.value2_hex_left) == u"11" else (u"c" if str(self.value2_hex_left) == u"12" else (u"d" if str(self.value2_hex_left) == u"13" else (u"e" if str(self.value2_hex_left) == u"14" else (u"f" if str(self.value2_hex_left) == u"15" else str(self.value2_hex_left)))))))
            return getattr(self, '_m_value2_hex_left_digit', None)

        @property
        def value8(self):
            if hasattr(self, '_m_value8'):
                return self._m_value8

            self._m_value8 = (self.fm_transmitter_t_upper_four_bits << 4)
            return getattr(self, '_m_value8', None)

        @property
        def value9_hex_left_digit(self):
            if hasattr(self, '_m_value9_hex_left_digit'):
                return self._m_value9_hex_left_digit

            self._m_value9_hex_left_digit = (u"a" if str(self.value9_hex_left) == u"10" else (u"b" if str(self.value9_hex_left) == u"11" else (u"c" if str(self.value9_hex_left) == u"12" else (u"d" if str(self.value9_hex_left) == u"13" else (u"e" if str(self.value9_hex_left) == u"14" else (u"f" if str(self.value9_hex_left) == u"15" else str(self.value9_hex_left)))))))
            return getattr(self, '_m_value9_hex_left_digit', None)

        @property
        def minus_x_t(self):
            if hasattr(self, '_m_minus_x_t'):
                return self._m_minus_x_t

            self._m_minus_x_t = (((((self.minus_x_t_upper_four_bits << 4) + 298.36) / 149.82) * 105.82) - 287.89)
            return getattr(self, '_m_minus_x_t', None)

        @property
        def value7_hex(self):
            if hasattr(self, '_m_value7_hex'):
                return self._m_value7_hex

            self._m_value7_hex = self.value7_hex_left_digit
            return getattr(self, '_m_value7_hex', None)

        @property
        def value2_hex(self):
            if hasattr(self, '_m_value2_hex'):
                return self._m_value2_hex

            self._m_value2_hex = self.value2_hex_left_digit
            return getattr(self, '_m_value2_hex', None)

        @property
        def value4_hex(self):
            if hasattr(self, '_m_value4_hex'):
                return self._m_value4_hex

            self._m_value4_hex = self.value4_hex_left_digit
            return getattr(self, '_m_value4_hex', None)

        @property
        def value9_hex_right(self):
            if hasattr(self, '_m_value9_hex_right'):
                return self._m_value9_hex_right

            self._m_value9_hex_right = (self.value9 % 16)
            return getattr(self, '_m_value9_hex_right', None)

        @property
        def value3_hex_left_digit(self):
            if hasattr(self, '_m_value3_hex_left_digit'):
                return self._m_value3_hex_left_digit

            self._m_value3_hex_left_digit = (u"a" if str(self.value3_hex_left) == u"10" else (u"b" if str(self.value3_hex_left) == u"11" else (u"c" if str(self.value3_hex_left) == u"12" else (u"d" if str(self.value3_hex_left) == u"13" else (u"e" if str(self.value3_hex_left) == u"14" else (u"f" if str(self.value3_hex_left) == u"15" else str(self.value3_hex_left)))))))
            return getattr(self, '_m_value3_hex_left_digit', None)

        @property
        def value9_hex_right_digit(self):
            if hasattr(self, '_m_value9_hex_right_digit'):
                return self._m_value9_hex_right_digit

            self._m_value9_hex_right_digit = (u"a" if str(self.value9_hex_right) == u"10" else (u"b" if str(self.value9_hex_right) == u"11" else (u"c" if str(self.value9_hex_right) == u"12" else (u"d" if str(self.value9_hex_right) == u"13" else (u"e" if str(self.value9_hex_right) == u"14" else (u"f" if str(self.value9_hex_right) == u"15" else str(self.value9_hex_right)))))))
            return getattr(self, '_m_value9_hex_right_digit', None)

        @property
        def value6_hex_left_digit(self):
            if hasattr(self, '_m_value6_hex_left_digit'):
                return self._m_value6_hex_left_digit

            self._m_value6_hex_left_digit = (u"a" if str(self.value6_hex_left) == u"10" else (u"b" if str(self.value6_hex_left) == u"11" else (u"c" if str(self.value6_hex_left) == u"12" else (u"d" if str(self.value6_hex_left) == u"13" else (u"e" if str(self.value6_hex_left) == u"14" else (u"f" if str(self.value6_hex_left) == u"15" else str(self.value6_hex_left)))))))
            return getattr(self, '_m_value6_hex_left_digit', None)

        @property
        def plus_y_t(self):
            if hasattr(self, '_m_plus_y_t'):
                return self._m_plus_y_t

            self._m_plus_y_t = (((((self.minus_x_t_upper_four_bits << 4) + 296.79) / 149.13) * 104.96) - 287.44)
            return getattr(self, '_m_plus_y_t', None)

        @property
        def fm_transmitter_t(self):
            if hasattr(self, '_m_fm_transmitter_t'):
                return self._m_fm_transmitter_t

            self._m_fm_transmitter_t = (((((self.fm_transmitter_t_upper_four_bits << 4) + 299.60) / 150.17) * 106.91) - 290.31)
            return getattr(self, '_m_fm_transmitter_t', None)

        @property
        def value2_hex_left(self):
            if hasattr(self, '_m_value2_hex_left'):
                return self._m_value2_hex_left

            self._m_value2_hex_left = self.value2 // 16
            return getattr(self, '_m_value2_hex_left', None)

        @property
        def value8_hex(self):
            if hasattr(self, '_m_value8_hex'):
                return self._m_value8_hex

            self._m_value8_hex = self.value8_hex_left_digit
            return getattr(self, '_m_value8_hex', None)

        @property
        def value5(self):
            if hasattr(self, '_m_value5'):
                return self._m_value5

            self._m_value5 = (self.plus_z_t_upper_four_bits << 4)
            return getattr(self, '_m_value5', None)

        @property
        def value1_hex_left_digit(self):
            if hasattr(self, '_m_value1_hex_left_digit'):
                return self._m_value1_hex_left_digit

            self._m_value1_hex_left_digit = (u"a" if str(self.value1_hex_left) == u"10" else (u"b" if str(self.value1_hex_left) == u"11" else (u"c" if str(self.value1_hex_left) == u"12" else (u"d" if str(self.value1_hex_left) == u"13" else (u"e" if str(self.value1_hex_left) == u"14" else (u"f" if str(self.value1_hex_left) == u"15" else str(self.value1_hex_left)))))))
            return getattr(self, '_m_value1_hex_left_digit', None)


    class Ut2(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.time_counter = self._io.read_bits_int_be(24)
            self._io.align_to_byte()
            self.discard = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def value1(self):
            if hasattr(self, '_m_value1'):
                return self._m_value1

            self._m_value1 = (self.time_counter >> 16)
            return getattr(self, '_m_value1', None)

        @property
        def value3_hex_right_digit(self):
            if hasattr(self, '_m_value3_hex_right_digit'):
                return self._m_value3_hex_right_digit

            self._m_value3_hex_right_digit = (u"a" if str(self.value3_hex_right) == u"10" else (u"b" if str(self.value3_hex_right) == u"11" else (u"c" if str(self.value3_hex_right) == u"12" else (u"d" if str(self.value3_hex_right) == u"13" else (u"e" if str(self.value3_hex_right) == u"14" else (u"f" if str(self.value3_hex_right) == u"15" else str(self.value3_hex_right)))))))
            return getattr(self, '_m_value3_hex_right_digit', None)

        @property
        def value1_hex_left(self):
            if hasattr(self, '_m_value1_hex_left'):
                return self._m_value1_hex_left

            self._m_value1_hex_left = self.value1 // 16
            return getattr(self, '_m_value1_hex_left', None)

        @property
        def value2_hex_right(self):
            if hasattr(self, '_m_value2_hex_right'):
                return self._m_value2_hex_right

            self._m_value2_hex_right = (self.value2 % 16)
            return getattr(self, '_m_value2_hex_right', None)

        @property
        def value3_hex(self):
            if hasattr(self, '_m_value3_hex'):
                return self._m_value3_hex

            self._m_value3_hex = self.value3_hex_left_digit + self.value3_hex_right_digit
            return getattr(self, '_m_value3_hex', None)

        @property
        def value3_hex_right(self):
            if hasattr(self, '_m_value3_hex_right'):
                return self._m_value3_hex_right

            self._m_value3_hex_right = (self.value3 % 16)
            return getattr(self, '_m_value3_hex_right', None)

        @property
        def value1_hex_right_digit(self):
            if hasattr(self, '_m_value1_hex_right_digit'):
                return self._m_value1_hex_right_digit

            self._m_value1_hex_right_digit = (u"a" if str(self.value1_hex_right) == u"10" else (u"b" if str(self.value1_hex_right) == u"11" else (u"c" if str(self.value1_hex_right) == u"12" else (u"d" if str(self.value1_hex_right) == u"13" else (u"e" if str(self.value1_hex_right) == u"14" else (u"f" if str(self.value1_hex_right) == u"15" else str(self.value1_hex_right)))))))
            return getattr(self, '_m_value1_hex_right_digit', None)

        @property
        def beacon(self):
            if hasattr(self, '_m_beacon'):
                return self._m_beacon

            self._m_beacon = u"UT2 " + self.value1_hex + self.value2_hex + self.value3_hex
            return getattr(self, '_m_beacon', None)

        @property
        def discard_discard(self):
            if hasattr(self, '_m_discard_discard'):
                return self._m_discard_discard

            if len(self.discard) != 0:
                self._m_discard_discard = int(self.discard) // 0

            return getattr(self, '_m_discard_discard', None)

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type

            self._m_beacon_type = 2
            return getattr(self, '_m_beacon_type', None)

        @property
        def value2(self):
            if hasattr(self, '_m_value2'):
                return self._m_value2

            self._m_value2 = ((self.time_counter >> 8) & 255)
            return getattr(self, '_m_value2', None)

        @property
        def value3_hex_left(self):
            if hasattr(self, '_m_value3_hex_left'):
                return self._m_value3_hex_left

            self._m_value3_hex_left = self.value3 // 16
            return getattr(self, '_m_value3_hex_left', None)

        @property
        def value1_hex_right(self):
            if hasattr(self, '_m_value1_hex_right'):
                return self._m_value1_hex_right

            self._m_value1_hex_right = (self.value1 % 16)
            return getattr(self, '_m_value1_hex_right', None)

        @property
        def value3(self):
            if hasattr(self, '_m_value3'):
                return self._m_value3

            self._m_value3 = (self.time_counter & 255)
            return getattr(self, '_m_value3', None)

        @property
        def value1_hex(self):
            if hasattr(self, '_m_value1_hex'):
                return self._m_value1_hex

            self._m_value1_hex = self.value1_hex_left_digit + self.value1_hex_right_digit
            return getattr(self, '_m_value1_hex', None)

        @property
        def value2_hex_right_digit(self):
            if hasattr(self, '_m_value2_hex_right_digit'):
                return self._m_value2_hex_right_digit

            self._m_value2_hex_right_digit = (u"a" if str(self.value2_hex_right) == u"10" else (u"b" if str(self.value2_hex_right) == u"11" else (u"c" if str(self.value2_hex_right) == u"12" else (u"d" if str(self.value2_hex_right) == u"13" else (u"e" if str(self.value2_hex_right) == u"14" else (u"f" if str(self.value2_hex_right) == u"15" else str(self.value2_hex_right)))))))
            return getattr(self, '_m_value2_hex_right_digit', None)

        @property
        def value2_hex_left_digit(self):
            if hasattr(self, '_m_value2_hex_left_digit'):
                return self._m_value2_hex_left_digit

            self._m_value2_hex_left_digit = (u"a" if str(self.value2_hex_left) == u"10" else (u"b" if str(self.value2_hex_left) == u"11" else (u"c" if str(self.value2_hex_left) == u"12" else (u"d" if str(self.value2_hex_left) == u"13" else (u"e" if str(self.value2_hex_left) == u"14" else (u"f" if str(self.value2_hex_left) == u"15" else str(self.value2_hex_left)))))))
            return getattr(self, '_m_value2_hex_left_digit', None)

        @property
        def value2_hex(self):
            if hasattr(self, '_m_value2_hex'):
                return self._m_value2_hex

            self._m_value2_hex = self.value2_hex_left_digit + self.value2_hex_right_digit
            return getattr(self, '_m_value2_hex', None)

        @property
        def value3_hex_left_digit(self):
            if hasattr(self, '_m_value3_hex_left_digit'):
                return self._m_value3_hex_left_digit

            self._m_value3_hex_left_digit = (u"a" if str(self.value3_hex_left) == u"10" else (u"b" if str(self.value3_hex_left) == u"11" else (u"c" if str(self.value3_hex_left) == u"12" else (u"d" if str(self.value3_hex_left) == u"13" else (u"e" if str(self.value3_hex_left) == u"14" else (u"f" if str(self.value3_hex_left) == u"15" else str(self.value3_hex_left)))))))
            return getattr(self, '_m_value3_hex_left_digit', None)

        @property
        def value2_hex_left(self):
            if hasattr(self, '_m_value2_hex_left'):
                return self._m_value2_hex_left

            self._m_value2_hex_left = self.value2 // 16
            return getattr(self, '_m_value2_hex_left', None)

        @property
        def value1_hex_left_digit(self):
            if hasattr(self, '_m_value1_hex_left_digit'):
                return self._m_value1_hex_left_digit

            self._m_value1_hex_left_digit = (u"a" if str(self.value1_hex_left) == u"10" else (u"b" if str(self.value1_hex_left) == u"11" else (u"c" if str(self.value1_hex_left) == u"12" else (u"d" if str(self.value1_hex_left) == u"13" else (u"e" if str(self.value1_hex_left) == u"14" else (u"f" if str(self.value1_hex_left) == u"15" else str(self.value1_hex_left)))))))
            return getattr(self, '_m_value1_hex_left_digit', None)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Stars(KaitaiStruct):
    """:field rm: rm
    :field beacon_no: beacon_no
    :field satellite_time: satellite_time
    :field condition: condition
    :field rssi: rssi
    :field temperature_1: temperature_1
    :field temperature_2: temperature_2
    :field temperature_3: temperature_3
    :field mode: mode
    :field reset_times_of_com_system: reset_times_of_com_system
    :field receive_times_of_cdh: receive_times_of_cdh
    :field solarcell_current: solarcell_current
    :field solarcell_voltage: solarcell_voltage
    :field total_system_current: total_system_current
    :field total_voltage: total_voltage
    :field solarcell_voltage_cdh: solarcell_voltage_cdh
    :field total_voltage_cdh: total_voltage_cdh
    :field necessary_for_lengthcheck: necessary_for_lengthcheck
    :field beacon: beacon
    
    .. seealso::
       Source - http://stars.eng.shizuoka.ac.jp/english/CW_Telemetry_format.pdf
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.rm = (self._io.read_bytes(1)).decode(u"ASCII")
        if not  ((self.rm == u"r") or (self.rm == u"m")) :
            raise kaitaistruct.ValidationNotAnyOfError(self.rm, self._io, u"/seq/0")
        self.beacon_no = (self._io.read_bytes(1)).decode(u"ASCII")
        if not  ((self.beacon_no == u"2") or (self.beacon_no == u"3") or (self.beacon_no == u"4") or (self.beacon_no == u"5") or (self.beacon_no == u"6")) :
            raise kaitaistruct.ValidationNotAnyOfError(self.beacon_no, self._io, u"/seq/1")
        self.value1 = self._io.read_u1()
        self.value2 = self._io.read_u1()
        self.value3 = self._io.read_u1()
        self.value4 = self._io.read_u1()
        self.lengthcheck = (self._io.read_bytes_full()).decode(u"utf-8")

    @property
    def temperature_3(self):
        if hasattr(self, '_m_temperature_3'):
            return self._m_temperature_3

        if self.beacon_no == u"3":
            self._m_temperature_3 = self.value4

        return getattr(self, '_m_temperature_3', None)

    @property
    def total_voltage(self):
        if hasattr(self, '_m_total_voltage'):
            return self._m_total_voltage

        if self.beacon_no == u"5":
            self._m_total_voltage = (self.value4 * 0.05888)

        return getattr(self, '_m_total_voltage', None)

    @property
    def total_voltage_cdh(self):
        if hasattr(self, '_m_total_voltage_cdh'):
            return self._m_total_voltage_cdh

        if self.beacon_no == u"6":
            self._m_total_voltage_cdh = (((self.value3 << 8) | self.value4) * 0.05888)

        return getattr(self, '_m_total_voltage_cdh', None)

    @property
    def reset_times_of_com_system(self):
        if hasattr(self, '_m_reset_times_of_com_system'):
            return self._m_reset_times_of_com_system

        if self.beacon_no == u"4":
            self._m_reset_times_of_com_system = self.value2

        return getattr(self, '_m_reset_times_of_com_system', None)

    @property
    def value3_hex_right_digit(self):
        if hasattr(self, '_m_value3_hex_right_digit'):
            return self._m_value3_hex_right_digit

        self._m_value3_hex_right_digit = (u"a" if str(self.value3_hex_right) == u"10" else (u"b" if str(self.value3_hex_right) == u"11" else (u"c" if str(self.value3_hex_right) == u"12" else (u"d" if str(self.value3_hex_right) == u"13" else (u"e" if str(self.value3_hex_right) == u"14" else (u"f" if str(self.value3_hex_right) == u"15" else str(self.value3_hex_right)))))))
        return getattr(self, '_m_value3_hex_right_digit', None)

    @property
    def necessary_for_lengthcheck(self):
        if hasattr(self, '_m_necessary_for_lengthcheck'):
            return self._m_necessary_for_lengthcheck

        if len(self.lengthcheck) != 0:
            self._m_necessary_for_lengthcheck = int(self.lengthcheck) // 0

        return getattr(self, '_m_necessary_for_lengthcheck', None)

    @property
    def value1_hex_left(self):
        if hasattr(self, '_m_value1_hex_left'):
            return self._m_value1_hex_left

        self._m_value1_hex_left = self.value1 // 16
        return getattr(self, '_m_value1_hex_left', None)

    @property
    def satellite_time(self):
        if hasattr(self, '_m_satellite_time'):
            return self._m_satellite_time

        if self.beacon_no == u"2":
            self._m_satellite_time = (((self.value1 << 16) | (self.value2 << 8)) | self.value3)

        return getattr(self, '_m_satellite_time', None)

    @property
    def condition(self):
        if hasattr(self, '_m_condition'):
            return self._m_condition

        if self.beacon_no == u"2":
            self._m_condition = self.value4

        return getattr(self, '_m_condition', None)

    @property
    def value2_hex_right(self):
        if hasattr(self, '_m_value2_hex_right'):
            return self._m_value2_hex_right

        self._m_value2_hex_right = (self.value2 % 16)
        return getattr(self, '_m_value2_hex_right', None)

    @property
    def value3_hex(self):
        if hasattr(self, '_m_value3_hex'):
            return self._m_value3_hex

        self._m_value3_hex = self.value3_hex_left_digit + self.value3_hex_right_digit
        return getattr(self, '_m_value3_hex', None)

    @property
    def total_system_current(self):
        if hasattr(self, '_m_total_system_current'):
            return self._m_total_system_current

        if self.beacon_no == u"5":
            self._m_total_system_current = (self.value3 * 0.025138)

        return getattr(self, '_m_total_system_current', None)

    @property
    def rssi(self):
        if hasattr(self, '_m_rssi'):
            return self._m_rssi

        if self.beacon_no == u"3":
            self._m_rssi = self.value1 // 2

        return getattr(self, '_m_rssi', None)

    @property
    def value3_hex_right(self):
        if hasattr(self, '_m_value3_hex_right'):
            return self._m_value3_hex_right

        self._m_value3_hex_right = (self.value3 % 16)
        return getattr(self, '_m_value3_hex_right', None)

    @property
    def temperature_2(self):
        if hasattr(self, '_m_temperature_2'):
            return self._m_temperature_2

        if self.beacon_no == u"3":
            self._m_temperature_2 = self.value3

        return getattr(self, '_m_temperature_2', None)

    @property
    def value1_hex_right_digit(self):
        if hasattr(self, '_m_value1_hex_right_digit'):
            return self._m_value1_hex_right_digit

        self._m_value1_hex_right_digit = (u"a" if str(self.value1_hex_right) == u"10" else (u"b" if str(self.value1_hex_right) == u"11" else (u"c" if str(self.value1_hex_right) == u"12" else (u"d" if str(self.value1_hex_right) == u"13" else (u"e" if str(self.value1_hex_right) == u"14" else (u"f" if str(self.value1_hex_right) == u"15" else str(self.value1_hex_right)))))))
        return getattr(self, '_m_value1_hex_right_digit', None)

    @property
    def beacon(self):
        if hasattr(self, '_m_beacon'):
            return self._m_beacon

        self._m_beacon = self.rm + self.beacon_no + u" " + self.value1_hex + u" " + self.value2_hex + u" " + self.value3_hex + u" " + self.value4_hex
        return getattr(self, '_m_beacon', None)

    @property
    def solarcell_voltage_cdh(self):
        if hasattr(self, '_m_solarcell_voltage_cdh'):
            return self._m_solarcell_voltage_cdh

        if self.beacon_no == u"6":
            self._m_solarcell_voltage_cdh = (((self.value1 << 8) | self.value2) * 0.05888)

        return getattr(self, '_m_solarcell_voltage_cdh', None)

    @property
    def value3_hex_left(self):
        if hasattr(self, '_m_value3_hex_left'):
            return self._m_value3_hex_left

        self._m_value3_hex_left = self.value3 // 16
        return getattr(self, '_m_value3_hex_left', None)

    @property
    def receive_times_of_cdh(self):
        if hasattr(self, '_m_receive_times_of_cdh'):
            return self._m_receive_times_of_cdh

        if self.beacon_no == u"4":
            self._m_receive_times_of_cdh = self.value4

        return getattr(self, '_m_receive_times_of_cdh', None)

    @property
    def solarcell_voltage(self):
        if hasattr(self, '_m_solarcell_voltage'):
            return self._m_solarcell_voltage

        if self.beacon_no == u"5":
            self._m_solarcell_voltage = (self.value2 * 0.05888)

        return getattr(self, '_m_solarcell_voltage', None)

    @property
    def value4_hex_left_digit(self):
        if hasattr(self, '_m_value4_hex_left_digit'):
            return self._m_value4_hex_left_digit

        self._m_value4_hex_left_digit = (u"a" if str(self.value4_hex_left) == u"10" else (u"b" if str(self.value4_hex_left) == u"11" else (u"c" if str(self.value4_hex_left) == u"12" else (u"d" if str(self.value4_hex_left) == u"13" else (u"e" if str(self.value4_hex_left) == u"14" else (u"f" if str(self.value4_hex_left) == u"15" else str(self.value4_hex_left)))))))
        return getattr(self, '_m_value4_hex_left_digit', None)

    @property
    def value1_hex_right(self):
        if hasattr(self, '_m_value1_hex_right'):
            return self._m_value1_hex_right

        self._m_value1_hex_right = (self.value1 % 16)
        return getattr(self, '_m_value1_hex_right', None)

    @property
    def value4_hex_right(self):
        if hasattr(self, '_m_value4_hex_right'):
            return self._m_value4_hex_right

        self._m_value4_hex_right = (self.value4 % 16)
        return getattr(self, '_m_value4_hex_right', None)

    @property
    def value4_hex_right_digit(self):
        if hasattr(self, '_m_value4_hex_right_digit'):
            return self._m_value4_hex_right_digit

        self._m_value4_hex_right_digit = (u"a" if str(self.value4_hex_right) == u"10" else (u"b" if str(self.value4_hex_right) == u"11" else (u"c" if str(self.value4_hex_right) == u"12" else (u"d" if str(self.value4_hex_right) == u"13" else (u"e" if str(self.value4_hex_right) == u"14" else (u"f" if str(self.value4_hex_right) == u"15" else str(self.value4_hex_right)))))))
        return getattr(self, '_m_value4_hex_right_digit', None)

    @property
    def value1_hex(self):
        if hasattr(self, '_m_value1_hex'):
            return self._m_value1_hex

        self._m_value1_hex = self.value1_hex_left_digit + self.value1_hex_right_digit
        return getattr(self, '_m_value1_hex', None)

    @property
    def value2_hex_right_digit(self):
        if hasattr(self, '_m_value2_hex_right_digit'):
            return self._m_value2_hex_right_digit

        self._m_value2_hex_right_digit = (u"a" if str(self.value2_hex_right) == u"10" else (u"b" if str(self.value2_hex_right) == u"11" else (u"c" if str(self.value2_hex_right) == u"12" else (u"d" if str(self.value2_hex_right) == u"13" else (u"e" if str(self.value2_hex_right) == u"14" else (u"f" if str(self.value2_hex_right) == u"15" else str(self.value2_hex_right)))))))
        return getattr(self, '_m_value2_hex_right_digit', None)

    @property
    def value4_hex_left(self):
        if hasattr(self, '_m_value4_hex_left'):
            return self._m_value4_hex_left

        self._m_value4_hex_left = self.value4 // 16
        return getattr(self, '_m_value4_hex_left', None)

    @property
    def value2_hex_left_digit(self):
        if hasattr(self, '_m_value2_hex_left_digit'):
            return self._m_value2_hex_left_digit

        self._m_value2_hex_left_digit = (u"a" if str(self.value2_hex_left) == u"10" else (u"b" if str(self.value2_hex_left) == u"11" else (u"c" if str(self.value2_hex_left) == u"12" else (u"d" if str(self.value2_hex_left) == u"13" else (u"e" if str(self.value2_hex_left) == u"14" else (u"f" if str(self.value2_hex_left) == u"15" else str(self.value2_hex_left)))))))
        return getattr(self, '_m_value2_hex_left_digit', None)

    @property
    def temperature_1(self):
        if hasattr(self, '_m_temperature_1'):
            return self._m_temperature_1

        if self.beacon_no == u"3":
            self._m_temperature_1 = self.value2

        return getattr(self, '_m_temperature_1', None)

    @property
    def value2_hex(self):
        if hasattr(self, '_m_value2_hex'):
            return self._m_value2_hex

        self._m_value2_hex = self.value2_hex_left_digit + self.value2_hex_right_digit
        return getattr(self, '_m_value2_hex', None)

    @property
    def mode(self):
        if hasattr(self, '_m_mode'):
            return self._m_mode

        if self.beacon_no == u"4":
            self._m_mode = self.value1

        return getattr(self, '_m_mode', None)

    @property
    def value4_hex(self):
        if hasattr(self, '_m_value4_hex'):
            return self._m_value4_hex

        self._m_value4_hex = self.value4_hex_left_digit + self.value4_hex_right_digit
        return getattr(self, '_m_value4_hex', None)

    @property
    def value3_hex_left_digit(self):
        if hasattr(self, '_m_value3_hex_left_digit'):
            return self._m_value3_hex_left_digit

        self._m_value3_hex_left_digit = (u"a" if str(self.value3_hex_left) == u"10" else (u"b" if str(self.value3_hex_left) == u"11" else (u"c" if str(self.value3_hex_left) == u"12" else (u"d" if str(self.value3_hex_left) == u"13" else (u"e" if str(self.value3_hex_left) == u"14" else (u"f" if str(self.value3_hex_left) == u"15" else str(self.value3_hex_left)))))))
        return getattr(self, '_m_value3_hex_left_digit', None)

    @property
    def value2_hex_left(self):
        if hasattr(self, '_m_value2_hex_left'):
            return self._m_value2_hex_left

        self._m_value2_hex_left = self.value2 // 16
        return getattr(self, '_m_value2_hex_left', None)

    @property
    def solarcell_current(self):
        if hasattr(self, '_m_solarcell_current'):
            return self._m_solarcell_current

        if self.beacon_no == u"5":
            self._m_solarcell_current = (self.value1 * 0.007906)

        return getattr(self, '_m_solarcell_current', None)

    @property
    def value1_hex_left_digit(self):
        if hasattr(self, '_m_value1_hex_left_digit'):
            return self._m_value1_hex_left_digit

        self._m_value1_hex_left_digit = (u"a" if str(self.value1_hex_left) == u"10" else (u"b" if str(self.value1_hex_left) == u"11" else (u"c" if str(self.value1_hex_left) == u"12" else (u"d" if str(self.value1_hex_left) == u"13" else (u"e" if str(self.value1_hex_left) == u"14" else (u"f" if str(self.value1_hex_left) == u"15" else str(self.value1_hex_left)))))))
        return getattr(self, '_m_value1_hex_left_digit', None)





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Casaasat(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field frame_id: ax25_frame.ax25_info.frame_id
    :field checksum: ax25_frame.ax25_info.checksum
    :field tm_code: ax25_frame.ax25_info.tm_code
    :field count: ax25_frame.ax25_info.count
    :field values___date: ax25_frame.ax25_info.values.___.type_check.date
    :field values___dosim_x1: ax25_frame.ax25_info.values.___.type_check.dosim_x1
    :field values___dosim_x2: ax25_frame.ax25_info.values.___.type_check.dosim_x2
    :field values___dosim_z1: ax25_frame.ax25_info.values.___.type_check.dosim_z1
    :field values___dosim_z2: ax25_frame.ax25_info.values.___.type_check.dosim_z2
    :field values___dosim_y1: ax25_frame.ax25_info.values.___.type_check.dosim_y1
    :field values___dosim_y2: ax25_frame.ax25_info.values.___.type_check.dosim_y2
    :field values___field_plus_y: ax25_frame.ax25_info.values.___.type_check.field_plus_y
    :field values___field_plus_x: ax25_frame.ax25_info.values.___.type_check.field_plus_x
    :field values___field_minus_z: ax25_frame.ax25_info.values.___.type_check.field_minus_z
    :field values___raw_minus_y: ax25_frame.ax25_info.values.___.type_check.raw_minus_y
    :field values___raw_minus_x: ax25_frame.ax25_info.values.___.type_check.raw_minus_x
    :field values___raw_minus_z: ax25_frame.ax25_info.values.___.type_check.raw_minus_z
    :field values___gain: ax25_frame.ax25_info.values.___.type_check.gain
    
    .. seealso::
       Source - https://site.amsat-f.org/download/120557/?tmstv=1735221517
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Casaasat.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Casaasat.Ax25Header(self._io, self, self._root)
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Casaasat.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Casaasat.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Casaasat.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Casaasat.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Casaasat.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()
            self.pid = self._io.read_u1()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 31) >> 1)
            return getattr(self, '_m_ssid', None)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Casaasat.Callsign(_io__raw_callsign_ror, self, self._root)


    class Ax25InfoData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.frame_id = self._io.read_u2be()
            self.checksum = self._io.read_u4be()
            self.tm_code = self._io.read_u1()
            if not  ((self.tm_code == 138) or (self.tm_code == 139)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.tm_code, self._io, u"/types/ax25_info_data/seq/2")
            self.count = self._io.read_u1()
            self.values = []
            for i in range(self.count):
                self.values.append(Casaasat.Ax25InfoData.ValuesT(self._io, self, self._root))


        class ValuesT(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                _on = self.tm_code_switch
                if _on == 138:
                    self.type_check = Casaasat.Ax25InfoData.Dosimeter(self._io, self, self._root)
                elif _on == 139:
                    self.type_check = Casaasat.Ax25InfoData.MagneticField(self._io, self, self._root)

            @property
            def tm_code_switch(self):
                if hasattr(self, '_m_tm_code_switch'):
                    return self._m_tm_code_switch

                _pos = self._io.pos()
                self._io.seek(6)
                self._m_tm_code_switch = self._io.read_u1()
                self._io.seek(_pos)
                return getattr(self, '_m_tm_code_switch', None)


        class Dosimeter(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.date = self._io.read_u4be()
                self.dosim_x1 = self._io.read_u2be()
                self.dosim_x2 = self._io.read_u2be()
                self.dosim_z1 = self._io.read_u2be()
                self.dosim_z2 = self._io.read_u2be()
                self.dosim_y1 = self._io.read_u2be()
                self.dosim_y2 = self._io.read_u2be()


        class MagneticField(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.date = self._io.read_u4be()
                self.field_plus_y = self._io.read_u4be()
                self.field_plus_x = self._io.read_u4be()
                self.field_minus_z = self._io.read_u4be()
                self.raw_minus_y = self._io.read_u2be()
                self.raw_minus_x = self._io.read_u2be()
                self.raw_minus_z = self._io.read_u2be()
                self.gain = self._io.read_u1()







"""
decode a single binary frame
"""

from __future__ import print_function

import argparse
import json

from satnogsdecoders import decoder
from satnogsdecoders.decoder import kaitai_to_dict


def decode_frame_to_fields(decoder_name: str, raw_frame: bytes):
    """
    Decode raw frames and return fields as specified by the
    fields definition in 'doc'.
    """

    decoder_class = getattr(decoder, decoder_name.capitalize())
    frame = decoder_class.from_bytes(raw_frame)

    return decoder.get_fields(frame, empty=False)


def decode_frame_to_dict(decoder_name: str, raw_frame: bytes):
    """
    Decode raw frames into nested dicts, as specified by the
    kaitai struct itself.
    """
    decoder_class = getattr(decoder, decoder_name.capitalize())
    frame = decoder_class.from_bytes(raw_frame)

    return kaitai_to_dict(frame)


def decode_frame(decoder_name: str, raw_frame: bytes):
    """
    Deprecated.
    Use either `decode_frame_to_dict` or `decode_frame_to_fields` to specify
    more precisely the expected response.
    """
    return decode_frame_to_fields(decoder_name, raw_frame)


def main():
    """
    main entry point
    """

    parser = argparse.ArgumentParser(
        description='Decode a raw frame with the selected decoder '
        '(generated by Kaitai) and print its json representation, '
        'either of the telemetry frame directly, or the transformed '
        'result based on the fields specified in the struct docstring.')
    parser.add_argument('decoder_name',
                        type=str,
                        help='name of the decoder (e.g. siriussat)')
    parser.add_argument('raw_frame_file',
                        type=str,
                        help='Path to the file containing the raw frame')
    parser.add_argument('--raw-frame',
                        action='store_true',
                        help='Show the full decoded frame. '
                        'If not selected (default), show only selected fields '
                        'as specified by the docstring.')
    args = parser.parse_args()

    # Load the raw frame
    with open(args.raw_frame_file, 'rb') as rawfile:
        raw_frame = rawfile.read()

    if args.raw_frame:
        data = decode_frame_to_dict(args.decoder_name, raw_frame)
    else:
        data = decode_frame_to_fields(args.decoder_name, raw_frame)

    print(json.dumps(data, indent=4, sort_keys=False))



"""
Preprocessor used in kaitai struct to decode Base85 (ASCII85) encoded
data
"""
import base64


class B85decode(object):  # pylint: disable=too-few-public-methods
    """
    b85decode preprocessor class
    """

    def __init__(self):
        pass

    def decode(self, in_bindata):  # pylint: disable=no-self-use
        """
        b85decode method
        """
        bindata = bytearray(in_bindata)
        return bytearray(base64.a85decode(bindata))



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import satnogsdecoders.process


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Enso(KaitaiStruct):
    """:field callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field ssid_mask: ax25_frame.ax25_header.dest_ssid_raw.ssid_mask
    :field ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field src_callsign_raw_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid_raw_ssid_mask: ax25_frame.ax25_header.src_ssid_raw.ssid_mask
    :field src_ssid_raw_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field repeater_rpt_instance_rpt_callsign_raw_callsign: ax25_frame.ax25_header.repeater.rpt_instance.rpt_callsign_raw.callsign_ror.callsign
    :field repeater_rpt_instance_rpt_ssid_raw_ssid_mask: ax25_frame.ax25_header.repeater.rpt_instance.rpt_ssid_raw.ssid_mask
    :field repeater_rpt_instance_rpt_ssid_raw_ssid: ax25_frame.ax25_header.repeater.rpt_instance.rpt_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field length: ax25_frame.payload.ax25_info.length
    :field frame_type: ax25_frame.payload.ax25_info.frame_type
    :field var_ts: ax25_frame.payload.ax25_info.var_ts
    :field obdh_timestamp: ax25_frame.payload.ax25_info.obdh_timestamp
    :field sat_mode: ax25_frame.payload.ax25_info.sat_mode
    :field obdh_mode: ax25_frame.payload.ax25_info.obdh_mode
    :field obdh_nb_bytes_to_transmit: ax25_frame.payload.ax25_info.obdh_nb_bytes_to_transmit
    :field obdh_nb_of_obdh_resets: ax25_frame.payload.ax25_info.obdh_nb_of_obdh_resets
    :field obdh_nb_of_errors: ax25_frame.payload.ax25_info.obdh_nb_of_errors
    :field eps_eps_mode: ax25_frame.payload.ax25_info.eps_eps_mode
    :field eps_bat_temp: ax25_frame.payload.ax25_info.eps_bat_temp
    :field eps_temp_z_minus: ax25_frame.payload.ax25_info.eps_temp_z_minus
    :field eps_obdh_cur: ax25_frame.payload.ax25_info.eps_obdh_cur
    :field eps_eps_cur: ax25_frame.payload.ax25_info.eps_eps_cur
    :field eps_ttc_micro_cur: ax25_frame.payload.ax25_info.eps_ttc_micro_cur
    :field eps_temp_x_plus: ax25_frame.payload.ax25_info.eps_temp_x_plus
    :field eps_temp_x_minus: ax25_frame.payload.ax25_info.eps_temp_x_minus
    :field eps_temp_y_plus: ax25_frame.payload.ax25_info.eps_temp_y_plus
    :field eps_temp_y_minus: ax25_frame.payload.ax25_info.eps_temp_y_minus
    :field eps_temp_z_plus: ax25_frame.payload.ax25_info.eps_temp_z_plus
    :field eps_temp_5v_reg: ax25_frame.payload.ax25_info.eps_temp_5v_reg
    :field eps_temp_6v_reg: ax25_frame.payload.ax25_info.eps_temp_6v_reg
    :field ttc_mode: ax25_frame.payload.ax25_info.ttc_mode
    :field ttc_resets: ax25_frame.payload.ax25_info.ttc_resets
    :field ttc_last_reset_cause: ax25_frame.payload.ax25_info.ttc_last_reset_cause
    :field ttc_total_valid_rcv_packets: ax25_frame.payload.ax25_info.ttc_total_valid_rcv_packets
    :field ttc_tx_packets: ax25_frame.payload.ax25_info.ttc_tx_packets
    :field ttc_power_fw_val: ax25_frame.payload.ax25_info.ttc_power_fw_val
    :field ttc_power_rev_val: ax25_frame.payload.ax25_info.ttc_power_rev_val
    :field ttc_last_err_code: ax25_frame.payload.ax25_info.ttc_last_err_code
    :field ttc_pwr_config: ax25_frame.payload.ax25_info.ttc_pwr_config
    :field ttc_pwr_amp_temp: ax25_frame.payload.ax25_info.ttc_pwr_amp_temp
    :field ttc_beacon_period: ax25_frame.payload.ax25_info.ttc_beacon_period
    :field tweet: ax25_frame.payload.ax25_info.tweet
    :field eps_bat_volt_val: ax25_frame.payload.ax25_info.eps_bat_volt_val
    :field eps_bat_volt_min_val: ax25_frame.payload.ax25_info.eps_bat_volt_min_val
    :field eps_bat_volt_max_val: ax25_frame.payload.ax25_info.eps_bat_volt_max_val
    :field eps_bat_volt_avg_val: ax25_frame.payload.ax25_info.eps_bat_volt_avg_val
    :field eps_avg_charge_cur_val: ax25_frame.payload.ax25_info.eps_avg_charge_cur_val
    :field eps_max_charge_cur_val: ax25_frame.payload.ax25_info.eps_max_charge_cur_val
    :field eps_ttc_tx_cur_val: ax25_frame.payload.ax25_info.eps_ttc_tx_cur_val
    :field eps_ttc_tx_cur_max_val: ax25_frame.payload.ax25_info.eps_ttc_tx_cur_max_val
    :field eps_pl_cur_val: ax25_frame.payload.ax25_info.eps_pl_cur_val
    :field eps_obdh_volt_val: ax25_frame.payload.ax25_info.eps_obdh_volt_val
    :field eps_ttc_volt_val: ax25_frame.payload.ax25_info.eps_ttc_volt_val
    :field eps_pl_volt_val: ax25_frame.payload.ax25_info.eps_pl_volt_val
    :field eps_mos1_volt_val: ax25_frame.payload.ax25_info.eps_mos1_volt_val
    :field eps_mos2_volt_val: ax25_frame.payload.ax25_info.eps_mos2_volt_val
    :field eps_mos3_volt_val: ax25_frame.payload.ax25_info.eps_mos3_volt_val
    :field eps_ref_volt_val: ax25_frame.payload.ax25_info.eps_ref_volt_val
    :field eps_ttc_mcu_volt_val: ax25_frame.payload.ax25_info.eps_ttc_mcu_volt_val
    :field ttc_rssi_last_packet_val: ax25_frame.payload.ax25_info.ttc_rssi_last_packet_val
    :field ttc_freq_dev_last_packet_val: ax25_frame.payload.ax25_info.ttc_freq_dev_last_packet_val
    :field temperature_val: ax25_frame.payload.ax25_info.temperature_val
    :field eps_charge_cur_val: ax25_frame.payload.ax25_info.eps_charge_cur_val
    :field payload: ax25_frame.payload.ax25_info.payload.payload_raw
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Enso.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Enso.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Enso.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Enso.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Enso.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Enso.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Enso.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Enso.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Enso.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Enso.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Enso.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Enso.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Enso.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Enso.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"F4KJX ") or (self.callsign == u"FX6FRC")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Enso.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class PayloadT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.payload_raw = (self._io.read_bytes(48)).decode(u"ASCII")


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Enso.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Enso.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Enso.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Enso.Callsign(_io__raw_callsign_ror, self, self._root)


    class Ax25InfoData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.length = self._io.read_u1()
            self.frame_type = self._io.read_u1()
            if not self.frame_type == 16:
                raise kaitaistruct.ValidationNotEqualError(16, self.frame_type, self._io, u"/types/ax25_info_data/seq/1")
            self.var_ts = self._io.read_u4le()
            self.obdh_timestamp = self._io.read_u4le()
            self.temperature = self._io.read_u2le()
            self.sat_mode = self._io.read_u1()
            self.obdh_mode = self._io.read_u1()
            self.obdh_nb_bytes_to_transmit = self._io.read_u4le()
            self.obdh_nb_of_obdh_resets = self._io.read_u2le()
            self.obdh_nb_of_errors = self._io.read_u2le()
            self.eps_eps_mode = self._io.read_u1()
            self.eps_bat_volt = self._io.read_u1()
            self.eps_bat_temp = self._io.read_s1()
            self.eps_bat_volt_min = self._io.read_u1()
            self.eps_bat_volt_max = self._io.read_u1()
            self.eps_bat_volt_avg = self._io.read_u1()
            self.eps_avg_charge_cur = self._io.read_u1()
            self.eps_max_charge_cur = self._io.read_u1()
            self.eps_temp_z_minus = self._io.read_s1()
            self.eps_obdh_cur = self._io.read_u1()
            self.eps_eps_cur = self._io.read_u1()
            self.eps_ttc_micro_cur = self._io.read_u1()
            self.eps_ttc_tx_cur = self._io.read_u1()
            self.eps_ttc_tx_cur_max = self._io.read_u1()
            self.eps_pl_cur = self._io.read_u1()
            self.eps_charge_cur = self._io.read_u1()
            self.eps_temp_x_plus = self._io.read_s1()
            self.eps_temp_x_minus = self._io.read_s1()
            self.eps_temp_y_plus = self._io.read_s1()
            self.eps_temp_y_minus = self._io.read_s1()
            self.eps_temp_z_plus = self._io.read_s1()
            self.eps_obdh_volt = self._io.read_u1()
            self.eps_ttc_volt = self._io.read_u1()
            self.eps_pl_volt = self._io.read_u1()
            self.eps_mos1_volt = self._io.read_u1()
            self.eps_mos2_volt = self._io.read_u1()
            self.eps_mos3_volt = self._io.read_u1()
            self.eps_ref_volt = self._io.read_u2le()
            self.eps_temp_5v_reg = self._io.read_s1()
            self.eps_temp_6v_reg = self._io.read_s1()
            self.eps_ttc_mcu_volt = self._io.read_u1()
            self.ttc_mode = self._io.read_u1()
            self.ttc_resets = self._io.read_u2le()
            self.ttc_last_reset_cause = self._io.read_u1()
            self.ttc_total_valid_rcv_packets = self._io.read_u2le()
            self.ttc_tx_packets = self._io.read_u2le()
            self.ttc_power_fw = self._io.read_u1()
            self.ttc_power_rev = self._io.read_u1()
            self.ttc_last_err_code = self._io.read_u1()
            self.ttc_pwr_config = self._io.read_u1()
            self.ttc_pwr_amp_temp = self._io.read_s1()
            self.ttc_rssi_last_packet = self._io.read_u1()
            self.ttc_freq_dev_last_packet = self._io.read_s1()
            self.ttc_beacon_period = self._io.read_u1()
            self._raw__raw_payload = self._io.read_bytes(48)
            _process = satnogsdecoders.process.Hexl()
            self._raw_payload = _process.decode(self._raw__raw_payload)
            _io__raw_payload = KaitaiStream(BytesIO(self._raw_payload))
            self.payload = Enso.PayloadT(_io__raw_payload, self, self._root)
            self.tweet = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def eps_ttc_mcu_volt_val(self):
            if hasattr(self, '_m_eps_ttc_mcu_volt_val'):
                return self._m_eps_ttc_mcu_volt_val

            self._m_eps_ttc_mcu_volt_val = ((self.eps_ttc_mcu_volt * 10) + 4000)
            return getattr(self, '_m_eps_ttc_mcu_volt_val', None)

        @property
        def eps_ttc_tx_cur_max_val(self):
            if hasattr(self, '_m_eps_ttc_tx_cur_max_val'):
                return self._m_eps_ttc_tx_cur_max_val

            self._m_eps_ttc_tx_cur_max_val = (self.eps_ttc_tx_cur_max * 5)
            return getattr(self, '_m_eps_ttc_tx_cur_max_val', None)

        @property
        def ttc_rssi_last_packet_val(self):
            if hasattr(self, '_m_ttc_rssi_last_packet_val'):
                return self._m_ttc_rssi_last_packet_val

            self._m_ttc_rssi_last_packet_val = (self.ttc_rssi_last_packet * -1)
            return getattr(self, '_m_ttc_rssi_last_packet_val', None)

        @property
        def eps_charge_cur_val(self):
            if hasattr(self, '_m_eps_charge_cur_val'):
                return self._m_eps_charge_cur_val

            self._m_eps_charge_cur_val = (self.eps_charge_cur * 12)
            return getattr(self, '_m_eps_charge_cur_val', None)

        @property
        def eps_max_charge_cur_val(self):
            if hasattr(self, '_m_eps_max_charge_cur_val'):
                return self._m_eps_max_charge_cur_val

            self._m_eps_max_charge_cur_val = (self.eps_max_charge_cur * 12)
            return getattr(self, '_m_eps_max_charge_cur_val', None)

        @property
        def eps_ttc_tx_cur_val(self):
            if hasattr(self, '_m_eps_ttc_tx_cur_val'):
                return self._m_eps_ttc_tx_cur_val

            self._m_eps_ttc_tx_cur_val = (self.eps_ttc_tx_cur * 5)
            return getattr(self, '_m_eps_ttc_tx_cur_val', None)

        @property
        def temperature_val(self):
            if hasattr(self, '_m_temperature_val'):
                return self._m_temperature_val

            self._m_temperature_val = ((self.temperature * 0.0625) if self.temperature <= 4095 else ((self.temperature - 8192) * 0.0625))
            return getattr(self, '_m_temperature_val', None)

        @property
        def eps_ttc_volt_val(self):
            if hasattr(self, '_m_eps_ttc_volt_val'):
                return self._m_eps_ttc_volt_val

            self._m_eps_ttc_volt_val = ((self.eps_ttc_volt * 10) + 4000)
            return getattr(self, '_m_eps_ttc_volt_val', None)

        @property
        def eps_ref_volt_val(self):
            if hasattr(self, '_m_eps_ref_volt_val'):
                return self._m_eps_ref_volt_val

            self._m_eps_ref_volt_val = (self.eps_ref_volt * 0.805)
            return getattr(self, '_m_eps_ref_volt_val', None)

        @property
        def eps_bat_volt_max_val(self):
            if hasattr(self, '_m_eps_bat_volt_max_val'):
                return self._m_eps_bat_volt_max_val

            self._m_eps_bat_volt_max_val = (self.eps_bat_volt_max * 20)
            return getattr(self, '_m_eps_bat_volt_max_val', None)

        @property
        def ttc_power_rev_val(self):
            if hasattr(self, '_m_ttc_power_rev_val'):
                return self._m_ttc_power_rev_val

            self._m_ttc_power_rev_val = (self.ttc_power_rev * 10)
            return getattr(self, '_m_ttc_power_rev_val', None)

        @property
        def eps_mos2_volt_val(self):
            if hasattr(self, '_m_eps_mos2_volt_val'):
                return self._m_eps_mos2_volt_val

            self._m_eps_mos2_volt_val = ((self.eps_mos2_volt + 2200) * 0.805)
            return getattr(self, '_m_eps_mos2_volt_val', None)

        @property
        def eps_bat_volt_min_val(self):
            if hasattr(self, '_m_eps_bat_volt_min_val'):
                return self._m_eps_bat_volt_min_val

            self._m_eps_bat_volt_min_val = (self.eps_bat_volt_min * 20)
            return getattr(self, '_m_eps_bat_volt_min_val', None)

        @property
        def eps_bat_volt_avg_val(self):
            if hasattr(self, '_m_eps_bat_volt_avg_val'):
                return self._m_eps_bat_volt_avg_val

            self._m_eps_bat_volt_avg_val = (self.eps_bat_volt_avg * 20)
            return getattr(self, '_m_eps_bat_volt_avg_val', None)

        @property
        def eps_pl_cur_val(self):
            if hasattr(self, '_m_eps_pl_cur_val'):
                return self._m_eps_pl_cur_val

            self._m_eps_pl_cur_val = (self.eps_pl_cur * 5)
            return getattr(self, '_m_eps_pl_cur_val', None)

        @property
        def eps_obdh_volt_val(self):
            if hasattr(self, '_m_eps_obdh_volt_val'):
                return self._m_eps_obdh_volt_val

            self._m_eps_obdh_volt_val = ((self.eps_obdh_volt * 10) + 4000)
            return getattr(self, '_m_eps_obdh_volt_val', None)

        @property
        def eps_bat_volt_val(self):
            if hasattr(self, '_m_eps_bat_volt_val'):
                return self._m_eps_bat_volt_val

            self._m_eps_bat_volt_val = (self.eps_bat_volt * 20)
            return getattr(self, '_m_eps_bat_volt_val', None)

        @property
        def eps_mos1_volt_val(self):
            if hasattr(self, '_m_eps_mos1_volt_val'):
                return self._m_eps_mos1_volt_val

            self._m_eps_mos1_volt_val = ((self.eps_mos1_volt + 2200) * 0.805)
            return getattr(self, '_m_eps_mos1_volt_val', None)

        @property
        def eps_mos3_volt_val(self):
            if hasattr(self, '_m_eps_mos3_volt_val'):
                return self._m_eps_mos3_volt_val

            self._m_eps_mos3_volt_val = ((self.eps_mos3_volt + 2200) * 0.805)
            return getattr(self, '_m_eps_mos3_volt_val', None)

        @property
        def ttc_freq_dev_last_packet_val(self):
            if hasattr(self, '_m_ttc_freq_dev_last_packet_val'):
                return self._m_ttc_freq_dev_last_packet_val

            self._m_ttc_freq_dev_last_packet_val = (self.ttc_freq_dev_last_packet * 17)
            return getattr(self, '_m_ttc_freq_dev_last_packet_val', None)

        @property
        def ttc_power_fw_val(self):
            if hasattr(self, '_m_ttc_power_fw_val'):
                return self._m_ttc_power_fw_val

            self._m_ttc_power_fw_val = (self.ttc_power_fw * 10)
            return getattr(self, '_m_ttc_power_fw_val', None)

        @property
        def eps_avg_charge_cur_val(self):
            if hasattr(self, '_m_eps_avg_charge_cur_val'):
                return self._m_eps_avg_charge_cur_val

            self._m_eps_avg_charge_cur_val = (self.eps_avg_charge_cur * 12)
            return getattr(self, '_m_eps_avg_charge_cur_val', None)

        @property
        def eps_pl_volt_val(self):
            if hasattr(self, '_m_eps_pl_volt_val'):
                return self._m_eps_pl_volt_val

            self._m_eps_pl_volt_val = ((self.eps_pl_volt * 10) + 4000)
            return getattr(self, '_m_eps_pl_volt_val', None)



